@@ -29,21 +29,80 @@
 #include "libwpd_internal.h"
 #include <limits>
 
-WPDConfidence WP1Heuristics::isWP1FileFormat(WPXInputStream *input, bool partialContent)
+WPDPasswordMatch WP1Heuristics::verifyPassword(WPXInputStream *input, const char *password)
 {
+	if (!password)
+		return WPD_PASSWORD_MATCH_DONTKNOW;
+
+	input->seek(0, WPX_SEEK_SET);
+	WPXEncryption *encryption = 0;
 	try
 	{
+		if (readU8(input, 0) == 0xFE && readU8(input, 0) == 0xFF &&
+			readU8(input, 0) == 0x61 && readU8(input, 0) == 0x61)
+		{
+			encryption = new WPXEncryption(password, 6);
+			if (readU16(input, 0, true) == encryption->getCheckSum())
+			{
+				delete encryption;
+				return WPD_PASSWORD_MATCH_OK;
+			}
+			else
+			{
+				delete encryption;
+				return WPD_PASSWORD_MATCH_NONE;
+			}
+		}
+		else
+			return WPD_PASSWORD_MATCH_DONTKNOW;
+	}
+	catch (...)
+	{
+		if (encryption)
+			delete encryption;
+		return WPD_PASSWORD_MATCH_DONTKNOW;
+	}
+}
+
+WPDConfidence WP1Heuristics::isWP1FileFormat(WPXInputStream *input, const char *password)
+{
+	input->seek(0, WPX_SEEK_SET);
+	WPXEncryption *encryption = 0;
+
+	try
+	{
+		if (readU8(input, 0) == 0xFE && readU8(input, 0) == 0xFF &&
+			readU8(input, 0) == 0x61 && readU8(input, 0) == 0x61)
+		{
+			if (password)
+			{
+				encryption = new WPXEncryption(password, 6);
+				if (readU16(input, 0, true) != encryption->getCheckSum())
+				{
+					delete encryption;
+					return WPD_CONFIDENCE_SUPPORTED_ENCRYPTION;
+				}
+			}
+			else
+			{
+				if (readU16(input,0) != 0x0000)
+					return WPD_CONFIDENCE_SUPPORTED_ENCRYPTION;
+			}
+		}
+		
+		input->seek(0, WPX_SEEK_SET);
+		if (password && encryption)
+			input->seek(6, WPX_SEEK_SET);	
+
 		int functionGroupCount = 0;
 	
-		WPD_DEBUG_MSG(("WP1Heuristics::isWP1FileFormat()\n"));
-		input->seek(0, WPX_SEEK_SET);
 		WPD_DEBUG_MSG(("WP1Heuristics::isWP1FileFormat()\n"));
 	
 		while (!input->atEOS())
 		{
-			uint8_t readVal = readU8(input);
+			uint8_t readVal = readU8(input, encryption);
 
-			WPD_DEBUG_MSG(("WP1Heuristics, Offset 0x%.8x, value 0x%.2x\n", (unsigned int)input->tell() - 1, readVal));
+			WPD_DEBUG_MSG(("WP1Heuristics, Offset 0x%.8x, value 0x%.2x (%c)\n", (unsigned int)input->tell() - 1, readVal, readVal));
 		
 			if (readVal < (uint8_t)0x20)
 			{
@@ -60,6 +119,8 @@ WPDConfidence WP1Heuristics::isWP1FileFormat(WPXInputStream *input, bool partial
 			}
 			else if (readVal >= (uint8_t)0xFF)
 			{
+				if (encryption)
+					delete encryption;
 				// special codes that should not be found as separate functions
 				return WPD_CONFIDENCE_NONE;
 			}
@@ -77,33 +138,53 @@ WPDConfidence WP1Heuristics::isWP1FileFormat(WPXInputStream *input, bool partial
 					//   <function code>{function length}...{function length}<function code>
 					//   that we observed in variable length WP1 functions 
 				
-					unsigned long functionLength = readU32(input, true);
+					unsigned long functionLength = readU32(input, encryption, true);
 					if (functionLength > ((std::numeric_limits<uint32_t>::max)() / 2))
+					{
+						if (encryption)
+							delete encryption;
 						return WPD_CONFIDENCE_NONE;
+					}
 					if (functionLength == 0)
+					{
+						if (encryption)
+							delete encryption;
 						return WPD_CONFIDENCE_NONE;
+					}
 					unsigned long closingFunctionLength = 0;
 					WPD_DEBUG_MSG(("WP1Heuristics functionLength = 0x%.8x\n", (unsigned int)functionLength));
 
 					input->seek(functionLength, WPX_SEEK_CUR);
-					closingFunctionLength = readU32(input, true);
+					closingFunctionLength = readU32(input, encryption, true);
 					WPD_DEBUG_MSG(("WP1Heuristics closingFunctionLength = 0x%.8x\n", (unsigned int)closingFunctionLength));
 					if (functionLength != closingFunctionLength)
+					{
+						if (encryption)
+							delete encryption;
 						return WPD_CONFIDENCE_NONE;
+					}
 					
 					uint8_t closingGate = 0;
 					if (!input->atEOS())
 					{
-						closingGate = readU8(input);
+						closingGate = readU8(input, encryption);
 						WPD_DEBUG_MSG(("WP1Heuristics closingGate = 0x%.2x\n", closingGate));
 						if (closingGate != readVal)
+						{
+							if (encryption)
+								delete encryption;
 							return WPD_CONFIDENCE_NONE;
+						}
 					}
 
 					// when passed the complete file, we don't allow for open groups when we've reached EOF
-					if (!partialContent && input->atEOS() && (closingGate != readVal))
+					if (input->atEOS() && (closingGate != readVal))
+					{
+						if (encryption)
+							delete encryption;
 						return WPD_CONFIDENCE_NONE;
-				
+					}
+
 					functionGroupCount++;
 				}
 				else
@@ -113,13 +194,21 @@ WPDConfidence WP1Heuristics::isWP1FileFormat(WPXInputStream *input, bool partial
 					// seek to the position where the closing gate should be
 					int res = input->seek(WP1_FUNCTION_GROUP_SIZE[readVal-0xC0]-2, WPX_SEEK_CUR);
 					// when passed the complete file, we should be able to do that
-					if (!partialContent && res)
+					if (res)
+					{
+						if (encryption)
+							delete encryption;
 						return WPD_CONFIDENCE_NONE;
-				
+					}
+
 					// read the closing gate
-					uint8_t readNextVal = readU8(input);
+					uint8_t readNextVal = readU8(input, encryption);
 					if (readNextVal != readVal)
+					{
+						if (encryption)
+							delete encryption;
 						return WPD_CONFIDENCE_NONE;
+					}
 				
 					functionGroupCount++;
 				}
@@ -131,12 +220,22 @@ WPDConfidence WP1Heuristics::isWP1FileFormat(WPXInputStream *input, bool partial
 		this would be the case when passed a plaintext file for example, which libwpd is not
 		supposed to handle. */
 		if (!functionGroupCount)
-			return WPD_CONFIDENCE_POOR;
-	
+		{
+			if (encryption)
+			{
+				delete encryption;
+				return WPD_CONFIDENCE_EXCELLENT;
+			}
+			return WPD_CONFIDENCE_NONE;
+		}
+		if (encryption)
+			delete encryption;
 		return WPD_CONFIDENCE_EXCELLENT;
 	}
 	catch (...)
 	{
+		if (encryption)
+			delete encryption;
 		return WPD_CONFIDENCE_NONE;
 	}
 }
