@@ -28,6 +28,7 @@
 #include "WPXFileStructure.h"
 #include "libwpd_internal.h"
 #include "WP3SubDocument.h"
+#include "WPDocument.h"
 #include <algorithm>
 #include <limits>
 
@@ -46,9 +47,9 @@ _WP3ContentParsingState::~_WP3ContentParsingState()
 	DELETEP(m_cellFillColor);
 }
 
-WP3ContentListener::WP3ContentListener(std::list<WPXPageSpan> &pageList, std::vector<WP3SubDocument *>&subDocuments, WPXHLListenerImpl *listenerImpl) :
+WP3ContentListener::WP3ContentListener(std::list<WPXPageSpan> &pageList, std::vector<WP3SubDocument *>&subDocuments, WPXDocumentInterface *documentInterface) :
 	WP3Listener(),
-	WPXContentListener(pageList, listenerImpl),
+	WPXContentListener(pageList, documentInterface),
 	m_parseState(new WP3ContentParsingState),
 	m_subDocuments(subDocuments)
 {
@@ -66,11 +67,12 @@ WP3ContentListener::~WP3ContentListener()
 
 void WP3ContentListener::insertCharacter(const uint16_t character)
 {
-        if (!isUndoOn())
+	if (!isUndoOn())
 	{
+		uint16_t tmpCharacter = _mapNonUnicodeCharacter(character);
 		if (!m_ps->m_isSpanOpened)
 			_openSpan();
-		appendUCS4(m_parseState->m_textBuffer, (uint32_t)character);
+		appendUCS4(m_parseState->m_textBuffer, (uint32_t)tmpCharacter);
 	}
 }
 
@@ -80,7 +82,7 @@ void WP3ContentListener::insertTab()
 	{
 		if (!m_ps->m_isParagraphOpened && !m_ps->m_isListElementOpened)
 		{
-			if (m_ps->m_tabStops.empty() || (_getNextTabStop() == (std::numeric_limits<float>::max)()))
+			if (m_ps->m_tabStops.empty() || (_getNextTabStop() == (std::numeric_limits<double>::max)()))
 				m_ps->m_textIndentByTabs += 0.5f;
 			else
 				m_ps->m_textIndentByTabs = _getNextTabStop() - (m_ps->m_leftMarginByTabs  + m_ps->m_textIndentByParagraphIndentChange);
@@ -99,12 +101,12 @@ void WP3ContentListener::insertTab()
 				_openSpan();
 			else
 				_flushText();
-			m_listenerImpl->insertTab();
+			m_documentInterface->insertTab();
 		}
 	}
 }
 
-void WP3ContentListener::insertTab(const uint8_t tabType, const float /* tabPosition */)
+void WP3ContentListener::insertTab(const uint8_t tabType, const double /* tabPosition */)
 {
 	if (!isUndoOn())
 	{
@@ -179,7 +181,7 @@ void WP3ContentListener::defineTable(const uint8_t position, const uint16_t left
 			break;
 		}
 		// Note: WordPerfect has an offset from the left edge of the page. We translate it to the offset from the left margin
-		m_ps->m_tableDefinition.m_leftOffset = _movePositionToFirstColumn( (float)((double)leftOffset / (double)WPX_NUM_WPUS_PER_INCH) ) - m_ps->m_paragraphMarginLeft;
+		m_ps->m_tableDefinition.m_leftOffset = _movePositionToFirstColumn( (double)((double)leftOffset / (double)WPX_NUM_WPUS_PER_INCH) ) - m_ps->m_paragraphMarginLeft;
 
 		// remove all the old column information
 		m_ps->m_tableDefinition.m_columns.clear();
@@ -195,9 +197,9 @@ void WP3ContentListener::addTableColumnDefinition(const uint32_t width, const ui
 	{
 		// define the new column
 		WPXColumnDefinition colDef;
-		colDef.m_width = (float)((double)width / (double)WPX_NUM_WPUS_PER_INCH);
-		colDef.m_leftGutter = (float)((double)width / (double)WPX_NUM_WPUS_PER_INCH);
-		colDef.m_rightGutter = (float)((double)width / (double)WPX_NUM_WPUS_PER_INCH);
+		colDef.m_width = (double)((double)width / (double)WPX_NUM_WPUS_PER_INCH);
+		colDef.m_leftGutter = (double)((double)width / (double)WPX_NUM_WPUS_PER_INCH);
+		colDef.m_rightGutter = (double)((double)width / (double)WPX_NUM_WPUS_PER_INCH);
 
 		// add the new column definition to our table definition
 		m_ps->m_tableDefinition.m_columns.push_back(colDef);
@@ -208,7 +210,7 @@ void WP3ContentListener::addTableColumnDefinition(const uint32_t width, const ui
 		
 		m_ps->m_tableDefinition.m_columnsProperties.push_back(colProp);
 		
-		// initialize the variable that tells us how many columns to skip
+		// initialize the variable that tells us how many row to skip
 		m_ps->m_numRowsToSkip.push_back(0);
 	}
 }
@@ -393,20 +395,20 @@ void WP3ContentListener::marginChange(const uint8_t side, const uint16_t margin)
 {
 	if (!isUndoOn())
 	{
-		float marginInch = (float)((double)margin/ (double)WPX_NUM_WPUS_PER_INCH);
+		double marginInch = (double)((double)margin/ (double)WPX_NUM_WPUS_PER_INCH);
 
 		switch(side)
 		{
 		case WPX_LEFT:
 			if (m_ps->m_numColumns > 1)
 			{
-				m_ps->m_leftMarginByPageMarginChange = 0.0f;
+				m_ps->m_leftMarginByPageMarginChange = 0.0;
 				m_ps->m_sectionMarginLeft = marginInch - m_ps->m_pageMarginLeft;
 			}
 			else
 			{
 				m_ps->m_leftMarginByPageMarginChange = marginInch - m_ps->m_pageMarginLeft;
-				m_ps->m_sectionMarginLeft = 0.0f;
+				m_ps->m_sectionMarginLeft = 0.0;
 			}
 			m_ps->m_paragraphMarginLeft = m_ps->m_leftMarginByPageMarginChange
 						+ m_ps->m_leftMarginByParagraphMarginChange
@@ -415,13 +417,13 @@ void WP3ContentListener::marginChange(const uint8_t side, const uint16_t margin)
 		case WPX_RIGHT:
 			if (m_ps->m_numColumns > 1)
 			{
-				m_ps->m_rightMarginByPageMarginChange = 0.0f;
+				m_ps->m_rightMarginByPageMarginChange = 0.0;
 				m_ps->m_sectionMarginRight = marginInch - m_ps->m_pageMarginRight;
 			}
 			else
 			{
 				m_ps->m_rightMarginByPageMarginChange = marginInch - m_ps->m_pageMarginRight;
-				m_ps->m_sectionMarginRight = 0.0f;
+				m_ps->m_sectionMarginRight = 0.0;
 			}
 			m_ps->m_paragraphMarginRight = m_ps->m_rightMarginByPageMarginChange
 						+ m_ps->m_rightMarginByParagraphMarginChange
@@ -466,7 +468,7 @@ void WP3ContentListener::indentFirstLineChange(const int16_t offset)
 {
 	if (!isUndoOn())
 	{
-		float offsetInch = (float)((double)offset / (double)WPX_NUM_WPUS_PER_INCH);
+		double offsetInch = (double)((double)offset / (double)WPX_NUM_WPUS_PER_INCH);
 		m_ps->m_textIndentByParagraphIndentChange = offsetInch;
 		// This is necessary in case we have Indent First Line and Hard Back Tab
 		// in the same time. The Hard Back Tab applies to the current paragraph
@@ -487,7 +489,7 @@ void WP3ContentListener::setTabs(const bool isRelative, const std::vector<WPXTab
 }
 
 void WP3ContentListener::columnChange(const WPXTextColumnType /* columnType */, const uint8_t numColumns,
-					const std::vector<float> &columnWidth, const std::vector<bool> &isFixedWidth)
+					const std::vector<double> &columnWidth, const std::vector<bool> &isFixedWidth)
 {
 	if (!isUndoOn())
 	{
@@ -500,7 +502,7 @@ void WP3ContentListener::columnChange(const WPXTextColumnType /* columnType */,
 		m_ps->m_isParagraphColumnBreak = false;
 		m_ps->m_isTextColumnWithoutParagraph = false;
 
-		float remainingSpace = m_ps->m_pageFormWidth - m_ps->m_pageMarginLeft - m_ps->m_sectionMarginLeft
+		double remainingSpace = m_ps->m_pageFormWidth - m_ps->m_pageMarginLeft - m_ps->m_sectionMarginLeft
 			- m_ps->m_pageMarginRight - m_ps->m_sectionMarginRight
 			- m_ps->m_leftMarginByPageMarginChange - m_ps->m_rightMarginByPageMarginChange;
 		// determine the space that is to be divided between columns whose width is expressed in percentage of remaining space
@@ -518,14 +520,14 @@ void WP3ContentListener::columnChange(const WPXTextColumnType /* columnType */,
 			for (i=0; i<numColumns; i++)
 			{
 				if (i == 0)
-					tmpColumn.m_leftGutter = 0.0f;
+					tmpColumn.m_leftGutter = 0.0;
 				else if (isFixedWidth[2*i-1])
 					tmpColumn.m_leftGutter = 0.5f * columnWidth[2*i-1];
 				else
 					tmpColumn.m_leftGutter = 0.5f * remainingSpace * columnWidth[2*i-1];
 				
 				if (i >= (numColumns - 1))
-					tmpColumn.m_rightGutter = 0.0f;
+					tmpColumn.m_rightGutter = 0.0;
 				else if (isFixedWidth[2*i+1])
 					tmpColumn.m_rightGutter = 0.5f * columnWidth[2*i+1];
 				else
@@ -590,7 +592,7 @@ void WP3ContentListener::setFontSize(const uint16_t fontSize)
 	{
 		_closeSpan();
 		
-		m_ps->m_fontSize=float(fontSize);
+		m_ps->m_fontSize=double(fontSize);
 	}
 }
 
@@ -612,7 +614,7 @@ void WP3ContentListener::insertNoteReference(const WPXString &noteReference)
 	}
 }
 
-void WP3ContentListener::insertNote(const WPXNoteType noteType, WP3SubDocument *subDocument)
+void WP3ContentListener::insertNote(const WPXNoteType noteType, const WP3SubDocument *subDocument)
 {
 	if (!isUndoOn() && !m_ps->m_isNote)
 	{
@@ -632,16 +634,16 @@ void WP3ContentListener::insertNote(const WPXNoteType noteType, WP3SubDocument *
 		propList.insert("libwpd:number", number);
 
 		if (noteType == FOOTNOTE)
-			m_listenerImpl->openFootnote(propList);
+			m_documentInterface->openFootnote(propList);
 		else
-			m_listenerImpl->openEndnote(propList);
+			m_documentInterface->openEndnote(propList);
 
-		handleSubDocument(subDocument, false, m_parseState->m_tableList, 0);
+		handleSubDocument(subDocument, WPX_SUBDOCUMENT_NOTE, m_parseState->m_tableList, 0);
 
 		if (noteType == FOOTNOTE)
-			m_listenerImpl->closeFootnote();
+			m_documentInterface->closeFootnote();
 		else
-			m_listenerImpl->closeEndnote();
+			m_documentInterface->closeEndnote();
 		m_ps->m_isNote = false;
 	}
 }
@@ -650,7 +652,7 @@ void WP3ContentListener::backTab()
 {
 	if (!isUndoOn() && !m_ps->m_isParagraphOpened && !m_ps->m_isListElementOpened)
 	{
-		if (m_ps->m_tabStops.empty() || (_getPreviousTabStop() == (std::numeric_limits<float>::max)()))
+		if (m_ps->m_tabStops.empty() || (_getPreviousTabStop() == (std::numeric_limits<double>::max)()))
 			m_ps->m_textIndentByTabs -= 0.5f;
 		else
 			m_ps->m_textIndentByTabs = _getPreviousTabStop() - (m_ps->m_leftMarginByTabs  + m_ps->m_textIndentByParagraphIndentChange);
@@ -672,12 +674,12 @@ void WP3ContentListener::leftIndent()
 	{
 		if (!m_ps->m_isParagraphOpened && !m_ps->m_isListElementOpened)
 		{
-			if (m_ps->m_tabStops.empty() || (_getNextTabStop() == (std::numeric_limits<float>::min)()))
+			if (m_ps->m_tabStops.empty() || (_getNextTabStop() == (std::numeric_limits<double>::min)()))
 				m_ps->m_leftMarginByTabs += 0.5f;
 			else
 				m_ps->m_leftMarginByTabs = _getNextTabStop() - (m_ps->m_textIndentByTabs + m_ps->m_textIndentByParagraphIndentChange);
 
-			if (m_ps->m_paragraphTextIndent != 0.0f)
+			if (m_ps->m_paragraphTextIndent != 0.0)
 				m_ps->m_textIndentByTabs -= m_ps->m_paragraphTextIndent;
 
 			m_ps->m_paragraphTextIndent = m_ps->m_textIndentByParagraphIndentChange
@@ -694,18 +696,18 @@ void WP3ContentListener::leftIndent()
 	}
 }
 
-void WP3ContentListener::leftIndent(const float offset)
+void WP3ContentListener::leftIndent(const double offset)
 {
 	if (!isUndoOn())
 	{
 		if (!m_ps->m_isParagraphOpened && !m_ps->m_isListElementOpened)
 		{
-			if (offset == 0.0f)
+			if (offset == 0.0)
 				m_ps->m_leftMarginByTabs += 0.5f;
 			else
-				m_ps->m_leftMarginByTabs += (offset / 72.0f);
+				m_ps->m_leftMarginByTabs += (offset / 72.0);
 
-			if (m_ps->m_paragraphTextIndent != 0.0f)
+			if (m_ps->m_paragraphTextIndent != 0.0)
 				m_ps->m_textIndentByTabs -= m_ps->m_paragraphTextIndent;
 
 			m_ps->m_paragraphTextIndent = m_ps->m_textIndentByParagraphIndentChange
@@ -728,12 +730,12 @@ void WP3ContentListener::leftRightIndent()
 	{
 		if (!m_ps->m_isParagraphOpened && !m_ps->m_isListElementOpened)
 		{
-			if (m_ps->m_tabStops.empty() || (_getNextTabStop() == (std::numeric_limits<float>::min)()))
+			if (m_ps->m_tabStops.empty() || (_getNextTabStop() == (std::numeric_limits<double>::min)()))
 				m_ps->m_leftMarginByTabs += 0.5f;
 			else
 				m_ps->m_leftMarginByTabs = _getNextTabStop() - (m_ps->m_textIndentByTabs + m_ps->m_textIndentByParagraphIndentChange);
 			
-			if (m_ps->m_paragraphTextIndent != 0.0f)
+			if (m_ps->m_paragraphTextIndent != 0.0)
 				m_ps->m_textIndentByTabs -= m_ps->m_paragraphTextIndent;
 
 			m_ps->m_rightMarginByTabs = m_ps->m_leftMarginByTabs;
@@ -752,18 +754,18 @@ void WP3ContentListener::leftRightIndent()
 	}
 }
 
-void WP3ContentListener::leftRightIndent(const float offset)
+void WP3ContentListener::leftRightIndent(const double offset)
 {
 	if (!isUndoOn())
 	{
 		if (!m_ps->m_isParagraphOpened && !m_ps->m_isListElementOpened)
 		{
-			if (offset == 0.0f)
+			if (offset == 0.0)
 				m_ps->m_leftMarginByTabs += 0.5f;
 			else
-				m_ps->m_leftMarginByTabs += (offset / 72.0f);
+				m_ps->m_leftMarginByTabs += (offset / 72.0);
 
-			if (m_ps->m_paragraphTextIndent != 0.0f)
+			if (m_ps->m_paragraphTextIndent != 0.0)
 				m_ps->m_textIndentByTabs -= m_ps->m_paragraphTextIndent;
 
 			m_ps->m_rightMarginByTabs = m_ps->m_leftMarginByTabs;
@@ -782,18 +784,346 @@ void WP3ContentListener::leftRightIndent(const float offset)
 	}
 }
 
-void WP3ContentListener::_handleSubDocument(const WPXSubDocument *subDocument, const bool isHeaderFooter,
+void WP3ContentListener::insertPicture(double height, double width, double verticalOffset, double horizontalOffset, uint8_t leftColumn, uint8_t rightColumn,
+			uint16_t figureFlags, const WPXBinaryData &binaryData)
+{
+	if (!isUndoOn())
+	{
+		if (!m_ps->m_isSpanOpened)
+			_openSpan();
+
+		WPXPropertyList propList;
+		_handleFrameParameters( propList, height, width, verticalOffset, horizontalOffset, leftColumn, rightColumn, figureFlags );
+		m_documentInterface->openFrame(propList);
+		
+		propList.clear();
+		propList.insert("libwpd:mimetype", "image/pict");
+		m_documentInterface->insertBinaryObject(propList, binaryData);
+		
+		m_documentInterface->closeFrame();
+	}
+}
+
+void WP3ContentListener::insertTextBox(double height, double width, double verticalOffset, double horizontalOffset, uint8_t leftColumn, uint8_t rightColumn,
+			uint16_t figureFlags, const WP3SubDocument * subDocument, const WP3SubDocument *caption)
+{
+	if (!isUndoOn())
+	{
+		if (!m_ps->m_isSpanOpened)
+			_openSpan();
+
+		WPXPropertyList propList;
+		_handleFrameParameters( propList, height, width, verticalOffset, horizontalOffset, leftColumn, rightColumn, figureFlags );
+		m_documentInterface->openFrame(propList);
+		
+		propList.clear();
+
+		if (subDocument || caption)
+		{
+			m_documentInterface->openTextBox(propList);
+		
+			// Positioned objects like text boxes are special beasts. They can contain all hierarchical elements up
+			// to the level of sections. They cannot open or close a page span though.
+			if (subDocument)
+				handleSubDocument(subDocument, WPX_SUBDOCUMENT_TEXT_BOX, m_parseState->m_tableList, 0);
+			if (caption)
+				handleSubDocument(caption, WPX_SUBDOCUMENT_TEXT_BOX, m_parseState->m_tableList, 0);
+
+			m_documentInterface->closeTextBox();
+		}
+		
+		m_documentInterface->closeFrame();
+	}
+}
+
+void WP3ContentListener::insertWP51Table(double height, double width, double verticalOffset, double horizontalOffset, uint8_t leftColumn, uint8_t rightColumn,
+			uint16_t figureFlags, const WP3SubDocument * subDocument, const WP3SubDocument *caption)
+{
+	if (!isUndoOn())
+	{
+		if (!m_ps->m_isSpanOpened)
+			_openSpan();
+
+		WPXPropertyList propList;
+		_handleFrameParameters( propList, height, width, verticalOffset, horizontalOffset, leftColumn, rightColumn, figureFlags );
+		m_documentInterface->openFrame(propList);
+
+		propList.clear();
+
+		if (subDocument || caption)
+		{
+			m_documentInterface->openTextBox(propList);
+		
+			// Positioned objects like text boxes are special beasts. They can contain all hierarchical elements up
+			// to the level of sections. They cannot open or close a page span though.
+			if (subDocument)
+				WPDocument::parseSubDocument(subDocument->getStream(), m_documentInterface, WPD_FILE_FORMAT_WP5);
+
+			if (caption)
+				handleSubDocument(caption, WPX_SUBDOCUMENT_TEXT_BOX, m_parseState->m_tableList, 0);
+
+			m_documentInterface->closeTextBox();
+		}
+
+		m_documentInterface->closeFrame();
+	}
+}
+
+void WP3ContentListener::_handleFrameParameters( WPXPropertyList &propList, double height, double width, double verticalOffset, double horizontalOffset,
+		uint8_t /* leftColumn */, uint8_t /* rightColumn */, uint16_t figureFlags  )
+{
+	propList.insert("svg:width", (double)((double)width/72.0));
+	propList.insert("svg:height", (double)((double)height/72.0));
+	
+	if ( figureFlags & 0x0080 )
+		propList.insert( "style:wrap", "dynamic" );
+	else
+		propList.insert( "style:wrap", "none" );
+
+	if ( ( figureFlags & 0x0300 ) == 0x0000 ) // paragraph
+	{
+		propList.insert("text:anchor-type", "paragraph");
+		propList.insert("style:vertical-rel", "paragraph" );
+		propList.insert("style:horizontal-rel", "paragraph");
+		switch ( figureFlags & 0x0003 )
+		{
+		case 0x01:
+			if (horizontalOffset == 0.0)
+				propList.insert("style:horizontal-pos", "right");
+			else
+			{
+				propList.insert( "style:horizontal-pos", "from-left");
+				propList.insert( "svg:x", (double)((double)horizontalOffset/72.0 - (double)width/72.0 +
+					(double)(m_ps->m_pageFormWidth - m_ps->m_pageMarginLeft - m_ps->m_pageMarginRight - m_ps->m_sectionMarginLeft
+					- m_ps->m_sectionMarginRight - m_ps->m_paragraphMarginLeft - m_ps->m_paragraphMarginRight)));
+			}
+			break;
+		case 0x02:
+			if (horizontalOffset == 0.0)
+				propList.insert("style:horizontal-pos", "center");
+			else
+			{
+				propList.insert( "style:horizontal-pos", "from-left");
+				propList.insert( "svg:x", (double)((double)horizontalOffset/72.0 - (double)width/(2.0*72.0) +
+					(double)(m_ps->m_pageFormWidth - m_ps->m_pageMarginLeft - m_ps->m_pageMarginRight - m_ps->m_sectionMarginLeft
+					- m_ps->m_sectionMarginRight - m_ps->m_paragraphMarginLeft - m_ps->m_paragraphMarginRight)/2.0));
+			}
+			break;
+		case 0x03:
+			propList.insert("svg:width", (double)(m_ps->m_pageFormWidth - m_ps->m_pageMarginLeft - m_ps->m_pageMarginRight - m_ps->m_sectionMarginLeft
+				- m_ps->m_sectionMarginRight - m_ps->m_paragraphMarginLeft - m_ps->m_paragraphMarginRight) );
+			propList.insert("style:horizontal-pos", "center");
+			break;
+		case 0x00:
+		default:
+			if (horizontalOffset == 0.0)
+				propList.insert("style:horizontal-pos", "left");
+			else
+			{
+				propList.insert( "style:horizontal-pos", "from-left");
+				propList.insert( "svg:x", (double)((double)horizontalOffset/72.0));
+			}
+			break;
+		}
+
+		if (verticalOffset == 0.0)
+			propList.insert( "style:vertical-pos", "top" );
+		else
+		{
+			propList.insert( "style:vertical-pos", "from-top" );
+			propList.insert( "svg:y", (double)((double)verticalOffset/72.0));
+		}
+
+	}
+	else if ( ( figureFlags & 0x0300 ) == 0x0100 ) // page
+	{
+		propList.insert("text:anchor-type", "char");
+		
+		if ( ( figureFlags & 0x1f08 ) == 0x0100 ) // full page
+		{
+			propList.insert("svg:width", (double)(m_ps->m_pageFormWidth - m_ps->m_pageMarginLeft - m_ps->m_pageMarginRight ) );
+			propList.insert("svg:height", (double)(m_ps->m_pageFormLength - m_ps->m_pageMarginTop - m_ps->m_pageMarginBottom ) );
+			propList.insert("style:vertical-rel", "page-content" );
+			propList.insert("style:vertical-pos", "middle" );
+			propList.insert("style:horizontal-rel", "page-content" );
+			propList.insert("style:horizontal-pos", "center" );
+		}
+		else if ( ( figureFlags & 0x1f08 ) == 0x1108 ) // abs. page
+		{
+			propList.insert("style:vertical-rel", "page" );
+			propList.insert("style:vertical-pos", "from-top" );
+			propList.insert("svg:y", (double)((double)verticalOffset/72.0));
+			propList.insert("style:horizontal-rel", "page-start-margin" );
+			propList.insert("style:horizontal-pos", "from-left" );
+			propList.insert("svg:x", (double)((double)horizontalOffset/72.0));
+		}
+		else
+		{
+			propList.insert("style:vertical-rel", "page-content" );
+			propList.insert("style:horizontal-rel", "page-content" );
+			switch ( ( figureFlags & 0x1c00 ) >> 10 )
+			{
+			case 0x00:
+				propList.insert("svg:height", (double)(m_ps->m_pageFormLength - m_ps->m_pageMarginTop - m_ps->m_pageMarginBottom ) );
+				propList.insert("style:vertical-rel", "page-content" );
+				propList.insert("style:vertical-pos", "middle" );
+				break;
+			case 0x01:
+				if ( verticalOffset == 0.0)
+					propList.insert("style:vertical-pos", "top" );
+				else
+				{
+					propList.insert("style:vertical-pos", "from-top" );
+					double newPosition = (double)((double)verticalOffset/72.0);
+					if (newPosition > (double)(m_ps->m_pageFormLength - m_ps->m_pageMarginTop - m_ps->m_pageMarginBottom
+						- (double)height/72.0) )
+						newPosition = (double)(m_ps->m_pageFormLength - m_ps->m_pageMarginTop - m_ps->m_pageMarginBottom
+						- (double)height/72.0);
+					propList.insert("svg:y", newPosition);
+				}
+				break;
+			case 0x02:
+				if (verticalOffset == 0.0)
+					propList.insert("style:vertical-pos", "middle" );
+				else
+				{
+					propList.insert("style:vertical-pos", "from-top" );
+					double newPosition = (double)((m_ps->m_pageFormLength - m_ps->m_pageMarginTop - m_ps->m_pageMarginBottom
+						- (double)height/72.0)/2.0);
+					if (newPosition > (double)(m_ps->m_pageFormLength - m_ps->m_pageMarginTop - m_ps->m_pageMarginBottom
+						- (double)height/72.0) )
+						newPosition = (double)(m_ps->m_pageFormLength - m_ps->m_pageMarginTop - m_ps->m_pageMarginBottom
+						- (double)height/72.0);
+					propList.insert("svg:y", newPosition);
+				}
+				break;
+			case 0x03:
+				if (verticalOffset == 0.0)
+					propList.insert("style:vertical-pos", "bottom" );
+				else
+				{
+					propList.insert("style:vertical-pos", "from-top" );
+					double newPosition = (double)(m_ps->m_pageFormLength - m_ps->m_pageMarginTop - m_ps->m_pageMarginBottom
+						- (double)height/72.0 + (double)verticalOffset/72.0);
+					if (newPosition > (double)(m_ps->m_pageFormLength - m_ps->m_pageMarginTop - m_ps->m_pageMarginBottom
+						- (double)height/72.0) )
+						newPosition = (double)(m_ps->m_pageFormLength - m_ps->m_pageMarginTop - m_ps->m_pageMarginBottom
+						- (double)height/72.0);
+					propList.insert("svg:y", newPosition);
+				}
+				break;
+			case 0x04:
+				propList.insert("style:vertical-rel", "page" );
+				propList.insert("style:vertical-pos", "from-top" );
+				propList.insert("svg:y", (double)((double)verticalOffset/72.0));
+				break;		
+			default:
+				break;
+			}
+			
+			switch ( figureFlags & 0x0003 )
+			{
+			case 0x00:
+				if ( horizontalOffset == 0.0 )
+					propList.insert( "style:horizontal-pos", "left");
+				else
+				{
+					propList.insert( "style:horizontal-pos", "from-left");
+					propList.insert( "svg:x", (double)((double)horizontalOffset/72.0));
+				}
+				break;
+			case 0x01:
+				if ( horizontalOffset == 0.0 )
+					propList.insert( "style:horizontal-pos", "right");
+				else
+				{
+					propList.insert( "style:horizontal-pos", "from-left");
+					propList.insert( "svg:x", (double)(m_ps->m_pageFormWidth - m_ps->m_pageMarginLeft - m_ps->m_pageMarginRight
+						- (double)width/72.0 + (double)horizontalOffset/72.0));
+				}
+				break;
+			case 0x02:
+				if ( horizontalOffset == 0.0 )
+					propList.insert( "style:horizontal-pos", "center" );
+				else
+				{
+					propList.insert( "style:horizontal-pos", "from-left");
+					propList.insert( "svg:x", (double)((m_ps->m_pageFormWidth - m_ps->m_pageMarginLeft - m_ps->m_pageMarginRight
+						- (double)width/72.0)/2.0 + (double)horizontalOffset/72.0));
+				}
+				break;
+			case 0x03:
+				propList.insert("svg:width", (double)(m_ps->m_pageFormWidth - m_ps->m_pageMarginLeft - m_ps->m_pageMarginRight ) );
+				propList.insert("style:horizontal-rel", "page-content" );
+				propList.insert("style:horizontal-pos", "center" );
+				break;
+			default:
+				break;
+			}
+		}
+	}
+	else if ( ( figureFlags & 0x0300 ) == 0x0200 ) // character
+	{
+		propList.insert("text:anchor-type", "as-char");
+		if ( ( figureFlags & 0x1c00 ) == 0x0000 )
+			propList.insert( "style:vertical-rel", "baseline" );
+		else
+			propList.insert( "style:vertical-rel", "line" );
+		switch ( ( figureFlags & 0x1c00 ) >> 10 )
+		{
+			case 0x01:
+				if ( verticalOffset == 0.0 )
+					propList.insert( "style:vertical-pos", "top" );
+				else
+				{
+					propList.insert( "style:vertical-pos", "from-top" );
+					propList.insert( "svg:y", (double)((double)verticalOffset/72.0));
+				}
+				break;
+			case 0x02:
+				if ( verticalOffset == 0.0 )
+					propList.insert( "style:vertical-pos", "middle" );
+				else
+				{
+					propList.insert( "style:vertical-pos", "from-top" );
+					propList.insert( "svg:y", (double)((double)verticalOffset/72.0 - (double)height/(2.0*72.0)));
+				}
+				break;
+			case 0x00:
+			case 0x03:
+				if ( verticalOffset == 0.0 )
+					propList.insert( "style:vertical-pos", "bottom" );
+				else
+				{
+					propList.insert( "style:vertical-pos", "from-top" );
+					propList.insert( "svg:y", (double)((double)verticalOffset/72.0 - (double)height/72.0));
+				}
+				break;
+			case 0x04:
+				break;
+			default:
+				break;
+		}
+	}
+}
+
+void WP3ContentListener::_handleSubDocument(const WPXSubDocument *subDocument, WPXSubDocumentType subDocumentType, 
 						WPXTableList /* tableList */, int /* nextTableIndice */)
 {
 	// save our old parsing state on our "stack"
 	WP3ContentParsingState *oldParseState = m_parseState;
 
 	m_parseState = new WP3ContentParsingState();
+	if (subDocumentType == WPX_SUBDOCUMENT_TEXT_BOX || subDocumentType == WPX_SUBDOCUMENT_COMMENT_ANNOTATION)
+	{
+		m_ps->m_pageMarginRight = 0.0;
+		m_ps->m_pageMarginLeft = 0.0;
+	}
 
 	bool oldIsUndoOn = isUndoOn();
 	setUndoOn(false);
 
-	if (isHeaderFooter)
+	if (subDocumentType == WPX_SUBDOCUMENT_HEADER_FOOTER)
 	{
 		marginChange(WPX_LEFT, WPX_NUM_WPUS_PER_INCH);
 		marginChange(WPX_RIGHT, WPX_NUM_WPUS_PER_INCH);
@@ -852,7 +1182,6 @@ void WP3ContentListener::_openParagraph()
 
 void WP3ContentListener::_flushText()
 {
-	if (m_parseState->m_textBuffer.len())
-		m_listenerImpl->insertText(m_parseState->m_textBuffer);
+	_insertText(m_parseState->m_textBuffer);
 	m_parseState->m_textBuffer.clear();
 }
