@@ -33,11 +33,11 @@
 class WP3ExtendedCharacterGroup : public WP3FixedLengthGroup
 {
 public:
-	WP3ExtendedCharacterGroup(WPXInputStream *input, uint8_t groupID);	
+	WP3ExtendedCharacterGroup(WPXInputStream *input, WPXEncryption *encryption, uint8_t groupID);	
 	void parse(WP3Listener *listener);
 	
  protected:
-	void _readContents(WPXInputStream *input);
+	void _readContents(WPXInputStream *input, WPXEncryption *encryption);
 
  private:
 	uint8_t m_macCharacter;
