@@ -24,7 +24,9 @@
  */
 
 #include "WP3Listener.h"
+#include "WP3ResourceFork.h"
 
-WP3Listener::WP3Listener()
+WP3Listener::WP3Listener() :
+	m_resourceFork()
 {
 }
