@@ -33,8 +33,8 @@
 class WP3UnsupportedVariableLengthGroup : public WP3VariableLengthGroup
 {
 public:
-	WP3UnsupportedVariableLengthGroup(WPXInputStream *input);
-	void parse(WP3Listener *listener) {}
+	WP3UnsupportedVariableLengthGroup(WPXInputStream *input, WPXEncryption *encryption);
+	void parse(WP3Listener * /* listener */) {}
 };
 
 #endif /* WP3UNSUPPORTEDVARIABLELENGTHGROUP_H */
