@@ -30,7 +30,7 @@
 #include "WP42Listener.h"
 #include "WP42SubDocument.h"
 #include "WPXContentListener.h"
-#include "WPXHLListenerImpl.h"
+#include "WPXDocumentInterface.h"
 
 typedef struct _WP42ContentParsingState WP42ContentParsingState;
 struct _WP42ContentParsingState
@@ -43,22 +43,24 @@ struct _WP42ContentParsingState
 class WP42ContentListener : public WP42Listener, protected WPXContentListener
 {
 public:
-	WP42ContentListener(std::list<WPXPageSpan> &pageList, std::vector<WP42SubDocument *> &subDocuments, WPXHLListenerImpl *listenerImpl);
+	WP42ContentListener(std::list<WPXPageSpan> &pageList, std::vector<WP42SubDocument *> &subDocuments, WPXDocumentInterface *documentInterface);
 	~WP42ContentListener();
 
 	void startDocument() { WPXContentListener::startDocument(); }
-	void insertCharacter(const uint16_t character);
-	void insertTab(const uint8_t tabType, float tabPosition);
-	void insertBreak(const uint8_t breakType) { WPXContentListener::insertBreak(breakType); }
+	void startSubDocument() { WPXContentListener::startSubDocument(); }
+	void insertCharacter(uint16_t character);
+	void insertTab(uint8_t tabType, double tabPosition);
+	void insertBreak(uint8_t breakType) { WPXContentListener::insertBreak(breakType); }
 	void insertEOL();
-	void attributeChange(const bool isOn, const uint8_t attribute);
-	void marginReset(const uint8_t leftMargin, const uint8_t rightMargin);
-	void headerFooterGroup(const uint8_t headerFooterDefinition, WP42SubDocument *subDocument);
-	void suppressPageCharacteristics(const uint8_t /* suppressCode */) {}
+	void attributeChange(bool isOn, uint8_t attribute);
+	void marginReset(uint8_t leftMargin, uint8_t rightMargin);
+	void headerFooterGroup(uint8_t headerFooterDefinition, WP42SubDocument *subDocument);
+	void suppressPageCharacteristics(uint8_t /* suppressCode */) {}
 	void endDocument() { WPXContentListener::endDocument(); };
+	void endSubDocument() { WPXContentListener::endSubDocument(); };
 
 protected:
-	void _handleSubDocument(const WPXSubDocument *subDocument, const bool isHeaderFooter, WPXTableList tableList, int nextTableIndice = 0);
+	void _handleSubDocument(const WPXSubDocument *subDocument, WPXSubDocumentType subDocumentType, WPXTableList tableList, int nextTableIndice = 0);
 
 	void _flushText();
 	void _changeList() {};
