@@ -35,13 +35,13 @@ class WP42MultiByteFunctionGroup : public WP42Part
 	WP42MultiByteFunctionGroup(uint8_t group); // WP42MultiByteFunctionGroup should _never_ be constructed, only its inherited classes
 	virtual ~WP42MultiByteFunctionGroup() {}
 	
-	static WP42MultiByteFunctionGroup * constructMultiByteFunctionGroup(WPXInputStream *input, uint8_t group);
+	static WP42MultiByteFunctionGroup * constructMultiByteFunctionGroup(WPXInputStream *input, WPXEncryption *encryption, uint8_t group);
 
  protected:
-	void _read(WPXInputStream *input);
- 	virtual void _readContents(WPXInputStream *input) = 0;
+	void _read(WPXInputStream *input, WPXEncryption *encryption);
+ 	virtual void _readContents(WPXInputStream *input, WPXEncryption *encryption) = 0;
 
-	const uint8_t getGroup() const { return m_group; }
+	uint8_t getGroup() const { return m_group; }
 
  private:
 	uint8_t m_group;
