@@ -34,7 +34,8 @@
 _WP5ContentParsingState::_WP5ContentParsingState() :
 	m_textBuffer(),
 	m_noteReference(),
-	m_tableList()
+	m_tableList(),
+	m_isFrameOpened(false)
 {
 }
 
@@ -42,12 +43,12 @@ _WP5ContentParsingState::~_WP5ContentParsingState()
 {
 }
 
-WP5ContentListener::WP5ContentListener(std::list<WPXPageSpan> &pageList, std::vector<WP5SubDocument*> &subDocuments, WPXHLListenerImpl *listenerImpl) :
+WP5ContentListener::WP5ContentListener(std::list<WPXPageSpan> &pageList, std::vector<WP5SubDocument*> &subDocuments, WPXDocumentInterface *documentInterface) :
 	WP5Listener(),
-	WPXContentListener(pageList, listenerImpl),
+	WPXContentListener(pageList, documentInterface),
 	m_parseState(new WP5ContentParsingState),
 	m_subDocuments(subDocuments),
-	m_defaultFontSize(12.0f),
+	m_defaultFontSize(12.0),
 	m_defaultFontName("Times New Roman")
 {
 }
@@ -62,17 +63,18 @@ WP5ContentListener::~WP5ContentListener()
  public 'HLListenerImpl' functions
 *****************************************/
 
-void WP5ContentListener::insertCharacter(const uint16_t character)
+void WP5ContentListener::insertCharacter(uint16_t character)
 {
+	uint16_t tmpCharacter = _mapNonUnicodeCharacter(character);
 	if (!m_ps->m_isSpanOpened)
 		_openSpan();
-	appendUCS4(m_parseState->m_textBuffer, (uint32_t)character);
+	appendUCS4(m_parseState->m_textBuffer, (uint32_t)tmpCharacter);
 }
 
-void WP5ContentListener::insertTab(const uint8_t tabType, const float tabPosition)
+void WP5ContentListener::insertTab(uint8_t tabType, double tabPosition)
 {
 	bool tmpHasTabPositionInformation = true;
-	if (tabPosition >= (float)((double)0xFFFE/(double)WPX_NUM_WPUS_PER_INCH) || tabPosition == 0.0f)
+	if (tabPosition >= (double)((double)0xFFFE/(double)WPX_NUM_WPUS_PER_INCH) || tabPosition == 0.0)
 		tmpHasTabPositionInformation = false;
 
 	if (!isUndoOn())
@@ -83,10 +85,12 @@ void WP5ContentListener::insertTab(const uint8_t tabType, const float tabPositio
 		case WP5_TAB_GROUP_RIGHT_TAB:
 		case WP5_TAB_GROUP_DECIMAL_TAB:
 			if (!m_ps->m_isParagraphOpened && !m_ps->m_isListElementOpened)
+			{
 				if (m_ps->m_currentListLevel == 0)
 					_openParagraph();
 				else
 					_openListElement();
+			}
 			break;
 
 		default:
@@ -152,7 +156,7 @@ void WP5ContentListener::insertTab(const uint8_t tabType, const float tabPositio
 			case WP5_TAB_GROUP_FLUSH_RIGHT:
 			case WP5_TAB_GROUP_RIGHT_TAB:
 			case WP5_TAB_GROUP_DECIMAL_TAB:
-				m_listenerImpl->insertTab();
+				m_documentInterface->insertTab();
 				break;
 			
 			default:
@@ -162,10 +166,10 @@ void WP5ContentListener::insertTab(const uint8_t tabType, const float tabPositio
 	}
 }
 
-void WP5ContentListener::insertIndent(const uint8_t indentType, const float indentPosition)
+void WP5ContentListener::insertIndent(uint8_t indentType, double indentPosition)
 {
 	bool tmpHasIndentPositionInformation = true;
-	if (indentPosition >= (float)((double)0xFFFE/(double)WPX_NUM_WPUS_PER_INCH) || indentPosition == 0.0f)
+	if (indentPosition >= (double)((double)0xFFFE/(double)WPX_NUM_WPUS_PER_INCH) || indentPosition == 0.0)
 		tmpHasIndentPositionInformation = false;
 
 	if (!isUndoOn())
@@ -180,7 +184,7 @@ void WP5ContentListener::insertIndent(const uint8_t indentType, const float inde
 				else
 					m_ps->m_leftMarginByTabs = indentPosition - m_ps->m_pageMarginLeft - m_ps->m_sectionMarginRight
 						- m_ps->m_leftMarginByPageMarginChange - m_ps->m_leftMarginByParagraphMarginChange;
-				if (m_ps->m_paragraphTextIndent != 0.0f)
+				if (m_ps->m_paragraphTextIndent != 0.0)
 					m_ps->m_textIndentByTabs -= m_ps->m_paragraphTextIndent;
 				break;
 
@@ -191,7 +195,7 @@ void WP5ContentListener::insertIndent(const uint8_t indentType, const float inde
 					m_ps->m_leftMarginByTabs = indentPosition - m_ps->m_pageMarginLeft - m_ps->m_sectionMarginLeft
 						- m_ps->m_leftMarginByPageMarginChange - m_ps->m_leftMarginByParagraphMarginChange;
 				m_ps->m_rightMarginByTabs = m_ps->m_leftMarginByTabs;
-				if (m_ps->m_paragraphTextIndent != 0.0f)
+				if (m_ps->m_paragraphTextIndent != 0.0)
 					m_ps->m_textIndentByTabs -= m_ps->m_paragraphTextIndent;
 				break;	
 				
@@ -215,7 +219,7 @@ void WP5ContentListener::insertIndent(const uint8_t indentType, const float inde
 			else
 				_flushText();
 
-			m_listenerImpl->insertTab();
+			m_documentInterface->insertTab();
 		}
 	}
 }
@@ -233,7 +237,7 @@ void WP5ContentListener::insertEOL()
 	}
 }
 
-void WP5ContentListener::defineTable(const uint8_t position, const uint16_t leftOffset)
+void WP5ContentListener::defineTable(uint8_t position, uint16_t leftOffset)
 {
 	if (!isUndoOn())
 	{
@@ -259,7 +263,7 @@ void WP5ContentListener::defineTable(const uint8_t position, const uint16_t left
 			break;
 		}
 		// Note: WordPerfect has an offset from the left edge of the page. We translate it to the offset from the left margin
-		m_ps->m_tableDefinition.m_leftOffset = (float)((double)leftOffset / (double)WPX_NUM_WPUS_PER_INCH) - m_ps->m_paragraphMarginLeft;
+		m_ps->m_tableDefinition.m_leftOffset = (double)((double)leftOffset / (double)WPX_NUM_WPUS_PER_INCH) - m_ps->m_paragraphMarginLeft;
 
 		// remove all the old column information
 		m_ps->m_tableDefinition.m_columns.clear();
@@ -268,16 +272,16 @@ void WP5ContentListener::defineTable(const uint8_t position, const uint16_t left
 	}
 }
 
-void WP5ContentListener::addTableColumnDefinition(const uint32_t width, const uint32_t /* leftGutter */,
-						const uint32_t /* rightGutter */, const uint32_t attributes, const uint8_t alignment)
+void WP5ContentListener::addTableColumnDefinition(uint32_t width, uint32_t /* leftGutter */,
+						uint32_t /* rightGutter */, uint32_t attributes, uint8_t alignment)
 {
 	if (!isUndoOn())
 	{
 		// define the new column
 		WPXColumnDefinition colDef;
-		colDef.m_width = (float)((double)width / (double)WPX_NUM_WPUS_PER_INCH);
-		colDef.m_leftGutter = (float)((double)width / (double)WPX_NUM_WPUS_PER_INCH);
-		colDef.m_rightGutter = (float)((double)width / (double)WPX_NUM_WPUS_PER_INCH);
+		colDef.m_width = (double)((double)width / (double)WPX_NUM_WPUS_PER_INCH);
+		colDef.m_leftGutter = (double)((double)width / (double)WPX_NUM_WPUS_PER_INCH);
+		colDef.m_rightGutter = (double)((double)width / (double)WPX_NUM_WPUS_PER_INCH);
 
 		// add the new column definition to our table definition
 		m_ps->m_tableDefinition.m_columns.push_back(colDef);
@@ -314,20 +318,20 @@ void WP5ContentListener::startTable()
 	}
 }
 
-void WP5ContentListener::insertRow(const uint16_t rowHeight, const bool isMinimumHeight, const bool isHeaderRow)
+void WP5ContentListener::insertRow(uint16_t rowHeight, bool isMinimumHeight, bool isHeaderRow)
 {
 	if (!isUndoOn())
 	{
 		_flushText();
-		float rowHeightInch = (float)((double) rowHeight / (double)WPX_NUM_WPUS_PER_INCH);
+		double rowHeightInch = (double)((double) rowHeight / (double)WPX_NUM_WPUS_PER_INCH);
 		_openTableRow(rowHeightInch, isMinimumHeight, isHeaderRow);
 	}
 }
 
-void WP5ContentListener::insertCell(const uint8_t colSpan, const uint8_t rowSpan, const uint8_t borderBits,
+void WP5ContentListener::insertCell(uint8_t colSpan, uint8_t rowSpan, uint8_t borderBits,
 			const RGBSColor * cellFgColor, const RGBSColor * cellBgColor, 
-			const RGBSColor * cellBorderColor, const WPXVerticalAlignment cellVerticalAlignment, 
-			const bool useCellAttributes, const uint32_t cellAttributes)
+			const RGBSColor * cellBorderColor, WPXVerticalAlignment cellVerticalAlignment, 
+			bool useCellAttributes, uint32_t cellAttributes)
 {
 	if (!isUndoOn())
 	{
@@ -365,7 +369,7 @@ void WP5ContentListener::endTable()
  public 'parser' functions
 *****************************************/
 
-void WP5ContentListener::attributeChange(const bool isOn, const uint8_t attribute)
+void WP5ContentListener::attributeChange(bool isOn, uint8_t attribute)
 {
 	if (!isUndoOn())
 	{
@@ -437,20 +441,20 @@ void WP5ContentListener::marginChange(uint8_t side, uint16_t margin)
 {
 	if (!isUndoOn())
 	{
-		float marginInch = (float)((double)margin/ (double)WPX_NUM_WPUS_PER_INCH);
+		double marginInch = (double)((double)margin/ (double)WPX_NUM_WPUS_PER_INCH);
 
 		switch(side)
 		{
 		case WPX_LEFT:
 			if (m_ps->m_numColumns > 1)
 			{
-				m_ps->m_leftMarginByPageMarginChange = 0.0f;
+				m_ps->m_leftMarginByPageMarginChange = 0.0;
 				m_ps->m_sectionMarginLeft = marginInch - m_ps->m_pageMarginLeft;
 			}
 			else
 			{
 				m_ps->m_leftMarginByPageMarginChange = marginInch - m_ps->m_pageMarginLeft;
-				m_ps->m_sectionMarginLeft = 0.0f;
+				m_ps->m_sectionMarginLeft = 0.0;
 			}
 			m_ps->m_paragraphMarginLeft = m_ps->m_leftMarginByPageMarginChange
 						+ m_ps->m_leftMarginByParagraphMarginChange
@@ -459,13 +463,13 @@ void WP5ContentListener::marginChange(uint8_t side, uint16_t margin)
 		case WPX_RIGHT:
 			if (m_ps->m_numColumns > 1)
 			{
-				m_ps->m_rightMarginByPageMarginChange = 0.0f;
+				m_ps->m_rightMarginByPageMarginChange = 0.0;
 				m_ps->m_sectionMarginRight = marginInch - m_ps->m_pageMarginRight;
 			}
 			else
 			{
 				m_ps->m_rightMarginByPageMarginChange = marginInch - m_ps->m_pageMarginRight;
-				m_ps->m_sectionMarginRight = 0.0f;
+				m_ps->m_sectionMarginRight = 0.0;
 			}
 			m_ps->m_paragraphMarginRight = m_ps->m_rightMarginByPageMarginChange
 						+ m_ps->m_rightMarginByParagraphMarginChange
@@ -476,7 +480,7 @@ void WP5ContentListener::marginChange(uint8_t side, uint16_t margin)
 	}
 }
 
-void WP5ContentListener::characterColorChange(const uint8_t red, const uint8_t green, const uint8_t blue)
+void WP5ContentListener::characterColorChange(uint8_t red, uint8_t green, uint8_t blue)
 {
 	if (!isUndoOn())
 	{
@@ -487,7 +491,7 @@ void WP5ContentListener::characterColorChange(const uint8_t red, const uint8_t g
  	}
 }
 
-void WP5ContentListener::setFont(const WPXString &fontName, const float fontSize)
+void WP5ContentListener::setFont(const WPXString &fontName, double fontSize)
 {
 	if (!isUndoOn())
 	{
@@ -497,7 +501,7 @@ void WP5ContentListener::setFont(const WPXString &fontName, const float fontSize
 	}
 }
 
-void WP5ContentListener::setTabs(const std::vector<WPXTabStop> &tabStops, const uint16_t tabOffset)
+void WP5ContentListener::setTabs(const std::vector<WPXTabStop> &tabStops, uint16_t tabOffset)
 {
 	if (!isUndoOn())
 	{
@@ -515,7 +519,7 @@ void WP5ContentListener::insertNoteReference(const WPXString &noteReference)
 	}
 }
 
-void WP5ContentListener::insertNote(const WPXNoteType noteType, const WP5SubDocument *subDocument)
+void WP5ContentListener::insertNote(WPXNoteType noteType, const WP5SubDocument *subDocument)
 {
 	if (!isUndoOn())
 	{
@@ -535,22 +539,22 @@ void WP5ContentListener::insertNote(const WPXNoteType noteType, const WP5SubDocu
 		propList.insert("libwpd:number", number);
 
 		if (noteType == FOOTNOTE)
-			m_listenerImpl->openFootnote(propList);
+			m_documentInterface->openFootnote(propList);
 		else
-			m_listenerImpl->openEndnote(propList);
+			m_documentInterface->openEndnote(propList);
 
-		handleSubDocument(subDocument, false, m_parseState->m_tableList, 0);
+		handleSubDocument(subDocument, WPX_SUBDOCUMENT_NOTE, m_parseState->m_tableList, 0);
 
 		if (noteType == FOOTNOTE)
-			m_listenerImpl->closeFootnote();
+			m_documentInterface->closeFootnote();
 		else
-			m_listenerImpl->closeEndnote();
+			m_documentInterface->closeEndnote();
 		m_ps->m_isNote = false;
 	}
 }
 
-void WP5ContentListener::_handleSubDocument(const WPXSubDocument *subDocument, const bool isHeaderFooter,
-				WPXTableList /* tableList */, int /* nextTableIndice */)
+void WP5ContentListener::_handleSubDocument(const WPXSubDocument *subDocument, WPXSubDocumentType subDocumentType, 
+	WPXTableList /* tableList */, int /* nextTableIndice */)
 {
 	// save our old parsing state on our "stack"
 	WP5ContentParsingState *oldParseState = m_parseState;
@@ -558,7 +562,7 @@ void WP5ContentListener::_handleSubDocument(const WPXSubDocument *subDocument, c
 	m_parseState = new WP5ContentParsingState();
 	setFont(m_defaultFontName, m_defaultFontSize);
 
-	if (isHeaderFooter)
+	if (subDocumentType == WPX_SUBDOCUMENT_HEADER_FOOTER)
 	{
 		marginChange(WPX_LEFT, WPX_NUM_WPUS_PER_INCH);
 		marginChange(WPX_RIGHT, WPX_NUM_WPUS_PER_INCH);
@@ -586,19 +590,231 @@ void WP5ContentListener::_handleSubDocument(const WPXSubDocument *subDocument, c
 	m_parseState = oldParseState;
 }
 
-void WP5ContentListener::headerFooterGroup(const uint8_t /* headerFooterType */, const uint8_t /* occurenceBits */,
+void WP5ContentListener::headerFooterGroup(uint8_t /* headerFooterType */, uint8_t /* occurenceBits */,
 						WP5SubDocument *subDocument)
 {
 	if (subDocument)
 		m_subDocuments.push_back(subDocument);
 }	
 
-void WP5ContentListener::setDefaultFont(const WPXString &fontName, const float fontSize)
+void WP5ContentListener::setDefaultFont(const WPXString &fontName, double fontSize)
 {
 	m_defaultFontName = fontName;
 	m_defaultFontSize = fontSize;
 }
 
+void WP5ContentListener::boxOn(uint8_t positionAndType, uint8_t alignment, uint16_t width, uint16_t height, uint16_t x, uint16_t y)
+{
+/*
+Paragraph:
+	- Vertical position:
+		+ offset from top of paragraph
+	- Horizontal position:
+		+ Left
+		+ Right
+		+ Centre
+		+ Full
+
+Page:
+	- Vertical position:
+		+ Full Page
+		+ Top
+		+ Centre
+		+ Bottom
+		+ Set Position
+			* Offset from the top of page
+	- Horizontal position
+		+ Margins
+			* Left
+			* Right
+			* Centre
+			* Full
+		+ Columns (Which column)
+			* Left
+			* Right
+			* Centre
+			* Full
+		+ Set Position
+			* Offset from left of page
+
+Character:
+	- Vertical position:
+		+ Top
+		+ Centre
+		+ Bottom
+		+ Baseline
+	- Horizontal Position:
+		+ N/A
+*/
+
+	if (isUndoOn() || (m_ps->m_isTableOpened && !m_ps->m_isTableCellOpened))
+		return;
+		 
+	if (!m_ps->m_isSpanOpened)
+		_openSpan();
+	else
+		_flushText();
+
+	WPXPropertyList propList;
+
+	propList.insert("svg:height", (double)((double)height/(double)WPX_NUM_WPUS_PER_INCH));
+	propList.insert("svg:width", (double)((double)width/(double)WPX_NUM_WPUS_PER_INCH));
+
+	if ( alignment & 0x80 )
+		propList.insert( "style:wrap", "dynamic" );
+	else
+		propList.insert( "style:wrap", "none" );
+
+
+	switch (positionAndType & 0x03)
+	{
+	case 0x00: // Paragraph anchored
+		propList.insert("text:anchor-type", "paragraph");
+		break;
+	case 0x01: // Page anchored
+		propList.insert("text:anchor-type", "char"); // ugly workaround to remediate OOo's wrong implementation of ODF
+		break;
+	case 0x02: // Char
+		propList.insert("text:anchor-type", "as-char");
+		break;
+	default:
+		break;
+	}
+	
+	propList.insert("style:vertical-rel", "page-content");
+	switch ( (positionAndType &  0x1c) >> 2 )
+	{
+	case 0x00:
+		propList.insert("svg:height", (double)(m_ps->m_pageFormLength - m_ps->m_pageMarginTop - m_ps->m_pageMarginBottom ) );
+		propList.insert("style:vertical-rel", "page-content" );
+		propList.insert("style:vertical-pos", "middle" );
+		break;
+	case 0x01:
+		if ( y == 0.0)
+			propList.insert("style:vertical-pos", "top" );
+		else
+		{
+			propList.insert("style:vertical-pos", "from-top" );
+			double newPosition = (double)((double)y/(double)WPX_NUM_WPUS_PER_INCH);
+			if (newPosition > (double)(m_ps->m_pageFormLength - m_ps->m_pageMarginTop - m_ps->m_pageMarginBottom
+				- (double)height/(double)WPX_NUM_WPUS_PER_INCH) )
+			{
+				newPosition = (double)(m_ps->m_pageFormLength - m_ps->m_pageMarginTop - m_ps->m_pageMarginBottom
+				- (double)height/(double)WPX_NUM_WPUS_PER_INCH);
+			}
+			propList.insert("svg:y", newPosition);
+		}
+		break;
+	case 0x02:
+		if (y == 0.0)
+			propList.insert("style:vertical-pos", "middle" );
+		else
+		{
+			propList.insert("style:vertical-pos", "from-top" );
+			double newPosition = (double)((m_ps->m_pageFormLength - m_ps->m_pageMarginTop - m_ps->m_pageMarginBottom
+				- (double)height/(double)WPX_NUM_WPUS_PER_INCH)/2.0);
+			if (newPosition > (double)(m_ps->m_pageFormLength - m_ps->m_pageMarginTop - m_ps->m_pageMarginBottom
+				- (double)height/(double)WPX_NUM_WPUS_PER_INCH) )
+			{
+				newPosition = (double)(m_ps->m_pageFormLength - m_ps->m_pageMarginTop - m_ps->m_pageMarginBottom
+					- (double)height/(double)WPX_NUM_WPUS_PER_INCH);
+			}
+			propList.insert("svg:y", newPosition);
+		}
+		break;
+	case 0x03:
+		if (y == 0.0)
+			propList.insert("style:vertical-pos", "bottom" );
+		else
+		{
+			propList.insert("style:vertical-pos", "from-top" );
+			double newPosition = (double)(m_ps->m_pageFormLength - m_ps->m_pageMarginTop - m_ps->m_pageMarginBottom
+				- (double)height/(double)WPX_NUM_WPUS_PER_INCH + (double)y/(double)WPX_NUM_WPUS_PER_INCH);
+			if (newPosition > (double)(m_ps->m_pageFormLength - m_ps->m_pageMarginTop - m_ps->m_pageMarginBottom
+				- (double)height/(double)WPX_NUM_WPUS_PER_INCH) )
+			{
+				newPosition = (double)(m_ps->m_pageFormLength - m_ps->m_pageMarginTop - m_ps->m_pageMarginBottom
+					- (double)height/(double)WPX_NUM_WPUS_PER_INCH);
+			}
+			propList.insert("svg:y", newPosition);
+		}
+		break;
+	case 0x04:
+		propList.insert("style:vertical-rel", "page" );
+		propList.insert("style:vertical-pos", "from-top" );
+		propList.insert("svg:y", (double)((double)y/(double)WPX_NUM_WPUS_PER_INCH));
+		break;
+	default:
+		break;
+	}
+	
+	propList.insert("style:horizontal-rel", "page-content" );
+	switch ( alignment & 0x03 )
+	{
+	case 0x00:
+		if ( x == 0.0 )
+			propList.insert( "style:horizontal-pos", "left");
+		else
+		{
+			propList.insert( "style:horizontal-pos", "from-left");
+			propList.insert( "svg:x", (double)((double)x/(double)WPX_NUM_WPUS_PER_INCH));
+		}
+		break;
+	case 0x01:
+		if ( x == 0.0 )
+			propList.insert( "style:horizontal-pos", "right");
+		else
+		{
+			propList.insert( "style:horizontal-pos", "from-left");
+			propList.insert( "svg:x", (double)(m_ps->m_pageFormWidth - m_ps->m_pageMarginLeft - m_ps->m_pageMarginRight
+				- (double)width/(double)WPX_NUM_WPUS_PER_INCH + (double)x/(double)WPX_NUM_WPUS_PER_INCH));
+		}
+		break;
+	case 0x02:
+		if ( x == 0.0 )
+			propList.insert( "style:horizontal-pos", "center" );
+		else
+		{
+			propList.insert( "style:horizontal-pos", "from-left");
+			propList.insert( "svg:x", (double)((m_ps->m_pageFormWidth - m_ps->m_pageMarginLeft - m_ps->m_pageMarginRight
+				- (double)width/(double)WPX_NUM_WPUS_PER_INCH)/2.0 + (double)x/(double)WPX_NUM_WPUS_PER_INCH));
+		}
+		break;
+	case 0x03:
+		propList.insert("svg:width", (double)(m_ps->m_pageFormWidth - m_ps->m_pageMarginLeft - m_ps->m_pageMarginRight ) );
+		propList.insert("style:horizontal-rel", "page-content" );
+		propList.insert("style:horizontal-pos", "center" );
+		break;
+	default:
+		break;
+	}
+	
+	m_documentInterface->openFrame(propList);
+	m_parseState->m_isFrameOpened = true;
+}
+
+void WP5ContentListener::boxOff()
+{
+	if (!isUndoOn() && m_parseState->m_isFrameOpened)
+	{
+		m_documentInterface->closeFrame();
+		m_parseState->m_isFrameOpened = false;
+	}
+}
+
+void WP5ContentListener::insertGraphicsData(const WPXBinaryData *data)
+{
+	if (isUndoOn() || !m_parseState->m_isFrameOpened)
+		return;
+
+	if (data) 
+	{
+		WPXPropertyList propList;
+		propList.insert("libwpd:mimetype", "image/x-wpg");
+		m_documentInterface->insertBinaryObject(propList, *data);
+	}
+}
+
 
 /****************************************
  private functions
@@ -606,8 +822,7 @@ void WP5ContentListener::setDefaultFont(const WPXString &fontName, const float f
 
 void WP5ContentListener::_flushText()
 {
-	if (m_parseState->m_textBuffer.len())
-		m_listenerImpl->insertText(m_parseState->m_textBuffer);
+	_insertText(m_parseState->m_textBuffer);
 	m_parseState->m_textBuffer.clear();
 }
 
