@@ -38,8 +38,8 @@ WP5StylesListener::WP5StylesListener(std::list<WPXPageSpan> &pageList, WPXTableL
 	m_nextPage(),
 	m_tableList(tableList),
 	m_currentTable(0), 
-	m_tempMarginLeft(1.0f),
-	m_tempMarginRight(1.0f),
+	m_tempMarginLeft(1.0),
+	m_tempMarginRight(1.0),
 	m_currentPageHasContent(false),
 	m_isSubDocument(false),
 	m_subDocuments(subDocuments),
@@ -52,7 +52,12 @@ void WP5StylesListener::endDocument()
 	insertBreak(WPX_SOFT_PAGE_BREAK); // pretend we just had a soft page break (for the last page)
 }
 
-void WP5StylesListener::insertBreak(const uint8_t breakType)
+void WP5StylesListener::endSubDocument()
+{	
+	insertBreak(WPX_SOFT_PAGE_BREAK); // pretend we just had a soft page break (for the last page)
+}
+
+void WP5StylesListener::insertBreak(uint8_t breakType)
 {
 	if (m_isSubDocument)
 		return;
@@ -75,7 +80,7 @@ void WP5StylesListener::insertBreak(const uint8_t breakType)
 				if (m_pageListHardPageMark == m_pageList.end())
 					m_pageListHardPageMark--;
 			}
-			m_currentPage = WPXPageSpan(m_pageList.back(), 0.0f, 0.0f);
+			m_currentPage = WPXPageSpan(m_pageList.back(), 0.0, 0.0);
 			m_currentPage.setPageSpan(1);
 
 			for(std::vector<WPXHeaderFooter>::const_iterator HFiter = (m_nextPage.getHeaderFooterList()).begin();
@@ -85,7 +90,7 @@ void WP5StylesListener::insertBreak(const uint8_t breakType)
 				{
 					m_currentPage.setHeaderFooter((*HFiter).getType(), (*HFiter).getInternalType(),
 						(*HFiter).getOccurence(), (*HFiter).getSubDocument(), (*HFiter).getTableList());
-					_handleSubDocument((*HFiter).getSubDocument(), true, (*HFiter).getTableList());
+					_handleSubDocument((*HFiter).getSubDocument(), WPX_SUBDOCUMENT_HEADER_FOOTER, (*HFiter).getTableList());
 				}
 				else
 					m_currentPage.setHeaderFooter((*HFiter).getType(), (*HFiter).getInternalType(),
@@ -104,11 +109,11 @@ void WP5StylesListener::insertBreak(const uint8_t breakType)
 	//}
 }
 
-void WP5StylesListener::pageMarginChange(const uint8_t side, const uint16_t margin)
+void WP5StylesListener::pageMarginChange(uint8_t side, uint16_t margin)
 {
 	//if (!isUndoOn()) 
 	//{
-		float marginInch = (float)((double)margin / (double)WPX_NUM_WPUS_PER_INCH);
+		double marginInch = (double)((double)margin / (double)WPX_NUM_WPUS_PER_INCH);
 		switch(side)
 		{
 			case WPX_TOP:
@@ -121,12 +126,12 @@ void WP5StylesListener::pageMarginChange(const uint8_t side, const uint16_t marg
 	//}
 }
 
-void WP5StylesListener::pageFormChange(const uint16_t length, const uint16_t width, const WPXFormOrientation orientation)
+void WP5StylesListener::pageFormChange(uint16_t length, uint16_t width, WPXFormOrientation orientation)
 {
 	//if (!isUndoOn())
 	//{
-		float lengthInch = (float)((double)length / (double)WPX_NUM_WPUS_PER_INCH);
-		float widthInch = (float)((double)width / (double)WPX_NUM_WPUS_PER_INCH);
+		double lengthInch = (double)((double)length / (double)WPX_NUM_WPUS_PER_INCH);
+		double widthInch = (double)((double)width / (double)WPX_NUM_WPUS_PER_INCH);
 		if (!m_currentPageHasContent)
 		{
 			m_currentPage.setFormLength(lengthInch);
@@ -137,7 +142,7 @@ void WP5StylesListener::pageFormChange(const uint16_t length, const uint16_t wid
 }
 
 
-void WP5StylesListener::marginChange(const uint8_t side, const uint16_t margin)
+void WP5StylesListener::marginChange(uint8_t side, uint16_t margin)
 {
 	if (!isUndoOn()) 
 	{		
@@ -145,7 +150,7 @@ void WP5StylesListener::marginChange(const uint8_t side, const uint16_t margin)
 			return;
 
 		std::list<WPXPageSpan>::iterator Iter;
-		float marginInch = (float)((double)margin / (double)WPX_NUM_WPUS_PER_INCH);
+		double marginInch = (double)((double)margin / (double)WPX_NUM_WPUS_PER_INCH);
 		switch(side)
 		{
 			case WPX_LEFT:
@@ -182,7 +187,7 @@ void WP5StylesListener::marginChange(const uint8_t side, const uint16_t margin)
 
 }
 
-void WP5StylesListener::headerFooterGroup(const uint8_t headerFooterType, const uint8_t occurenceBits, WP5SubDocument *subDocument)
+void WP5StylesListener::headerFooterGroup(uint8_t headerFooterType, uint8_t occurenceBits, WP5SubDocument *subDocument)
 {
 	if (subDocument)
 		m_subDocuments.push_back(subDocument);
@@ -219,7 +224,7 @@ void WP5StylesListener::headerFooterGroup(const uint8_t headerFooterType, const
 				if (wpxOccurence != NEVER)
 				{
 					m_currentPage.setHeaderFooter(wpxType, headerFooterType, wpxOccurence, subDocument, tableList);
-					_handleSubDocument(subDocument, true, tableList);
+					_handleSubDocument(subDocument, WPX_SUBDOCUMENT_HEADER_FOOTER, tableList);
 				}
 				else
 					m_currentPage.setHeaderFooter(wpxType, headerFooterType, wpxOccurence, 0, tableList);
@@ -230,7 +235,7 @@ void WP5StylesListener::headerFooterGroup(const uint8_t headerFooterType, const
 }
 
 
-void WP5StylesListener::suppressPageCharacteristics(const uint8_t suppressCode)
+void WP5StylesListener::suppressPageCharacteristics(uint8_t suppressCode)
 {
 	if (!isUndoOn()) 
 	{			
@@ -256,7 +261,7 @@ void WP5StylesListener::startTable()
 	}
 }
 
-void WP5StylesListener::insertRow(const uint16_t /* rowHeight */, const bool /* isMinimumHeight */, const bool /* isHeaderRow */)
+void WP5StylesListener::insertRow(uint16_t /* rowHeight */, bool /* isMinimumHeight */, bool /* isHeaderRow */)
 {
 	if (!isUndoOn()) 
 	{
@@ -267,10 +272,10 @@ void WP5StylesListener::insertRow(const uint16_t /* rowHeight */, const bool /*
 	}
 }
 
-void WP5StylesListener::insertCell(const uint8_t colSpan, const uint8_t rowSpan, const uint8_t borderBits, 
+void WP5StylesListener::insertCell(uint8_t colSpan, uint8_t rowSpan, uint8_t borderBits, 
 				  const RGBSColor * /* cellFgColor */, const RGBSColor * /* cellBgColor */,
-				  const RGBSColor * /* cellBorderColor */, const WPXVerticalAlignment /* cellVerticalAlignment */, 
-				  const bool /* useCellAttributes */, const uint32_t /* cellAttributes */)
+				  const RGBSColor * /* cellBorderColor */, WPXVerticalAlignment /* cellVerticalAlignment */, 
+				  bool /* useCellAttributes */, uint32_t /* cellAttributes */)
 {
 	if (!isUndoOn())
 	{
@@ -281,7 +286,7 @@ void WP5StylesListener::insertCell(const uint8_t colSpan, const uint8_t rowSpan,
 	}
 }
 
-void WP5StylesListener::_handleSubDocument(const WPXSubDocument *subDocument, const bool isHeaderFooter,
+void WP5StylesListener::_handleSubDocument(const WPXSubDocument *subDocument, WPXSubDocumentType subDocumentType, 
 						WPXTableList tableList, int /* nextTableIndice */)
 {
 	// We don't want to actual insert anything in the case of a sub-document, but we
@@ -290,7 +295,7 @@ void WP5StylesListener::_handleSubDocument(const WPXSubDocument *subDocument, co
 	{
 		bool oldIsSubDocument = m_isSubDocument;
 		m_isSubDocument = true;
-		if (isHeaderFooter) 
+		if (subDocumentType == WPX_SUBDOCUMENT_HEADER_FOOTER) 
 		{
 			bool oldCurrentPageHasContent = m_currentPageHasContent;
 			WPXTable * oldCurrentTable = m_currentTable;
