@@ -30,7 +30,8 @@
 class WP60Header : public WP6Header
 {
  public:
-	WP60Header(WPXInputStream * input, uint32_t documentOffset, uint8_t productType, uint8_t fileType, uint8_t majorVersion, uint8_t minorVersion, uint16_t documentEncryption);
+	WP60Header(WPXInputStream * input, WPXEncryption *encryption, uint32_t documentOffset, uint8_t productType,
+		uint8_t fileType, uint8_t majorVersion, uint8_t minorVersion, uint16_t documentEncryption);
 	~WP60Header();
 };
 #endif /* _WP60HEADER_H  */
