@@ -27,10 +27,11 @@
 #include <ctype.h>
 #include "libwpd_math.h"
 #include "WP6ContentListener.h"
-#include "WPXHLListenerImpl.h"
+#include "WPXDocumentInterface.h"
 #include "WP6FileStructure.h"
 #include "WPXFileStructure.h"
 #include "WP6FontDescriptorPacket.h"
+#include "WP6GraphicsCachedFileDataPacket.h"
 #include "WP6DefaultInitialFontPacket.h"
 #include "libwpd_internal.h"
 #include "WP6SubDocument.h"
@@ -41,7 +42,7 @@
 #include <algorithm>
 #include <time.h>
 
-#define WP6_DEFAULT_FONT_SIZE 12.0f
+#define WP6_DEFAULT_FONT_SIZE 12.0
 #define WP6_DEFAULT_FONT_NAME "Times New Roman"
 
 WP6OutlineDefinition::WP6OutlineDefinition(const WP6OutlineLocation outlineLocation, const uint8_t *numberingMethods,
@@ -108,8 +109,8 @@ _WP6ContentParsingState::_WP6ContentParsingState(WPXTableList tableList, int nex
 	m_numberText(),
 	m_textAfterDisplayReference(),
 	m_textAfterNumber(),
-	m_paragraphMarginBottomRelative(1.0f),
-	m_paragraphMarginBottomAbsolute(0.0f),
+	m_paragraphMarginBottomRelative(1.0),
+	m_paragraphMarginBottomAbsolute(0.0),
 
 	m_numRemovedParagraphBreaks(0),
 	
@@ -133,11 +134,15 @@ _WP6ContentParsingState::_WP6ContentParsingState(WPXTableList tableList, int nex
 	m_noteTextPID(0),
 	m_numNestedNotes(0),
 
+	m_isFrameOpened(false),
+
 	m_leaderCharacter('.'),
 	m_leaderNumSpaces(0),
 
 	m_tempTabStops(),
-	m_tempUsePreWP9LeaderMethod()
+	m_tempUsePreWP9LeaderMethod(),
+
+	m_currentPageNumberingType(ARABIC)
 
 {
 }
@@ -147,9 +152,9 @@ _WP6ContentParsingState::~_WP6ContentParsingState()
 	// FIXME: erase current fontname
 }
 
-WP6ContentListener::WP6ContentListener(std::list<WPXPageSpan> &pageList, WPXTableList tableList, WPXHLListenerImpl *listenerImpl) :
+WP6ContentListener::WP6ContentListener(std::list<WPXPageSpan> &pageList, WPXTableList tableList, WPXDocumentInterface *documentInterface) :
 	WP6Listener(),
-	WPXContentListener(pageList, listenerImpl),
+	WPXContentListener(pageList, documentInterface),
 	m_parseState(new WP6ContentParsingState(tableList)),
 	m_outlineDefineHash()
 {
@@ -200,6 +205,7 @@ void WP6ContentListener::setDate(const uint16_t type, const uint16_t year,
         if (retVal <= 0) {
                 // error - date wouldn't fit in buffer
                 dateStr.sprintf("ERROR: %d character buffer too short for date", DATEBUFLEN);
+		return; // don't continue, because we cannot trust the data here
         }
         else {
                 dateStr.sprintf("%s", dateBuffer);
@@ -211,8 +217,6 @@ void WP6ContentListener::setDate(const uint16_t type, const uint16_t year,
 			m_metaData.insert("meta:creation-date", dateStr);
 			break;
 		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_DATE_COMPLETED):
-			// output two forms until we decide which to use
-			m_metaData.insert("libwpd:completed-date", dateStr);
 			m_metaData.insert("dcterms:available", dateStr);
 			break;
 		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_RECORDED_DATE):
@@ -226,8 +230,6 @@ void WP6ContentListener::setDate(const uint16_t type, const uint16_t year,
 		//	m_metaData.insert("libwpd:revision-date", dateStr);
 		//	break;
 		case (WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_VERSION_DATE):
-			// output two forms until we decide which to use
-			m_metaData.insert("libwpd:version-date", dateStr);
 			m_metaData.insert("dcterms:issued", dateStr);
 			break;
 	}
@@ -412,40 +414,42 @@ void WP6ContentListener::insertCharacter(const uint16_t character)
 {
 	if (!isUndoOn())
 	{
+		uint16_t tmpCharacter = _mapNonUnicodeCharacter(character);
+
 		if (m_parseState->m_styleStateSequence.getCurrentState() == STYLE_BODY ||
 		    m_parseState->m_styleStateSequence.getCurrentState() == NORMAL)
 		{
 			if (!m_ps->m_isSpanOpened)
 				_openSpan();
 			m_parseState->m_isListReference = false;
-			appendUCS4(m_parseState->m_bodyText, (uint32_t)character);
+			appendUCS4(m_parseState->m_bodyText, (uint32_t)tmpCharacter);
 		}
 		else if (m_parseState->m_styleStateSequence.getCurrentState() == BEGIN_BEFORE_NUMBERING)
 		{
 			m_parseState->m_isListReference = true;
-			appendUCS4(m_parseState->m_textBeforeNumber, (uint32_t)character);
+			appendUCS4(m_parseState->m_textBeforeNumber, (uint32_t)tmpCharacter);
 		}
 		else if (m_parseState->m_styleStateSequence.getCurrentState() == BEGIN_NUMBERING_BEFORE_DISPLAY_REFERENCING)
 		{
 			// left delimeter (or the bullet if there is no display referencing)
-			appendUCS4(m_parseState->m_textBeforeDisplayReference, (uint32_t)character);
+			appendUCS4(m_parseState->m_textBeforeDisplayReference, (uint32_t)tmpCharacter);
 			m_parseState->m_isListReference = true;
 		}
 		else if (m_parseState->m_styleStateSequence.getCurrentState() == DISPLAY_REFERENCING)
 		{
 			// the actual paragraph number (in varying forms)
-			appendUCS4(m_parseState->m_numberText, (uint32_t)character);
+			appendUCS4(m_parseState->m_numberText, (uint32_t)tmpCharacter);
 			m_parseState->m_isListReference = true;
 		}
 		else if (m_parseState->m_styleStateSequence.getCurrentState() == BEGIN_NUMBERING_AFTER_DISPLAY_REFERENCING)
 		{
 			// right delimeter (if there was a display no. ref. group)
-			appendUCS4(m_parseState->m_textAfterDisplayReference, (uint32_t)character);
+			appendUCS4(m_parseState->m_textAfterDisplayReference, (uint32_t)tmpCharacter);
 			m_parseState->m_isListReference = true;
 		}
 		else if (m_parseState->m_styleStateSequence.getCurrentState() == BEGIN_AFTER_NUMBERING)
 		{
-			appendUCS4(m_parseState->m_textAfterNumber, (uint32_t)character);
+			appendUCS4(m_parseState->m_textAfterNumber, (uint32_t)tmpCharacter);
 			m_parseState->m_isListReference = true;
 		}
 	}
@@ -468,10 +472,10 @@ void WP6ContentListener::defineTabStops(const bool isRelative, const std::vector
 }
 
 
-void WP6ContentListener::insertTab(const uint8_t tabType, float tabPosition)
+void WP6ContentListener::insertTab(const uint8_t tabType, double tabPosition)
 {
 	bool tmpHasTabPositionInformation = true;
-	if (tabPosition >= (float)((double)0xFFFE/(double)WPX_NUM_WPUS_PER_INCH))
+	if (tabPosition >= (double)((double)0xFFFE/(double)WPX_NUM_WPUS_PER_INCH))
 		tmpHasTabPositionInformation = false;
 	else
 		tabPosition = _movePositionToFirstColumn(tabPosition);
@@ -494,10 +498,12 @@ void WP6ContentListener::insertTab(const uint8_t tabType, float tabPosition)
 			case WP6_TAB_GROUP_RIGHT_TAB:
 			case WP6_TAB_GROUP_DECIMAL_TAB:
 				if (!m_ps->m_isParagraphOpened && !m_ps->m_isListElementOpened)
+				{
 					if (m_ps->m_currentListLevel == 0)
 						_openParagraph();
 					else
 						_openListElement();
+				}
 				break;
 
 			default:
@@ -557,7 +563,7 @@ void WP6ContentListener::insertTab(const uint8_t tabType, float tabPosition)
 						- m_ps->m_leftMarginByPageMarginChange - m_ps->m_leftMarginByParagraphMarginChange;
 				if (m_parseState->m_isListReference)
 					m_parseState->m_numListExtraTabs++;
-				if (m_ps->m_paragraphTextIndent != 0.0f)
+				if (m_ps->m_paragraphTextIndent != 0.0)
 					m_ps->m_textIndentByTabs -= m_ps->m_paragraphTextIndent;
 				break;
 
@@ -573,7 +579,7 @@ void WP6ContentListener::insertTab(const uint8_t tabType, float tabPosition)
 				// L/R Indent is symetrical from the effective paragraph margins and position indicates only
 				// the distance from the left edge
 				m_ps->m_rightMarginByTabs = m_ps->m_leftMarginByTabs;
-				if (m_ps->m_paragraphTextIndent != 0.0f)
+				if (m_ps->m_paragraphTextIndent != 0.0)
 					m_ps->m_textIndentByTabs -= m_ps->m_paragraphTextIndent;
 				break;	
 				
@@ -613,10 +619,10 @@ void WP6ContentListener::insertTab(const uint8_t tabType, float tabPosition)
 			case WP6_TAB_GROUP_FLUSH_RIGHT:
 			case WP6_TAB_GROUP_RIGHT_TAB:
 			case WP6_TAB_GROUP_DECIMAL_TAB:
-				m_listenerImpl->insertTab();
+				m_documentInterface->insertTab();
 				break;
 			case WP6_TAB_GROUP_BAR_TAB:
-				m_listenerImpl->insertTab();
+				m_documentInterface->insertTab();
 				insertCharacter('|');  // We emulate the bar tab
 				break;
 			
@@ -641,7 +647,7 @@ void WP6ContentListener::handleLineBreak()
 			else
 				_flushText();
 
-			m_listenerImpl->insertLineBreak();
+			m_documentInterface->insertLineBreak();
 		}
 	}
 }
@@ -692,28 +698,28 @@ void WP6ContentListener::highlightChange(const bool isOn, const RGBSColor color)
 	}
 }
 
-void WP6ContentListener::fontChange(const uint16_t matchedFontPointSize, const uint16_t fontPID)
+void WP6ContentListener::fontChange(const uint16_t matchedFontPointSize, const uint16_t fontPID, const WPXString &fontName)
 {
 	if (!isUndoOn())
 	{
 		_closeSpan();
 		if (matchedFontPointSize)
 		{
-			m_ps->m_fontSize = (float)rint((double)((((double)matchedFontPointSize)/100.0f)*2.0f));
+			m_ps->m_fontSize = wpuToFontPointSize(matchedFontPointSize);
 			// We compute the real space after paragraph in inches using the size of the font and relative spacing.
 			// We have to recompute this every change of fontSize.
 			m_ps->m_paragraphMarginBottom =
-				(float)(((m_parseState->m_paragraphMarginBottomRelative - 1.0f)*m_ps->m_fontSize)/72.0f) +
+				(double)(((m_parseState->m_paragraphMarginBottomRelative - 1.0)*m_ps->m_fontSize)/72.0) +
 				m_parseState->m_paragraphMarginBottomAbsolute;
 		}
 		if (fontPID)
 		{
-			const WP6FontDescriptorPacket *fontDescriptorPacket = 
-				dynamic_cast<const WP6FontDescriptorPacket *>(WP6Listener::getPrefixDataPacket(fontPID));
-			if (fontDescriptorPacket) {
-				*(m_ps->m_fontName) = fontDescriptorPacket->getFontName();
-			}
+			WPXString pidFontName = WP6Listener::getFontNameForPID(fontPID);
+			if (!!pidFontName)
+				*(m_ps->m_fontName) = pidFontName;
 		}
+		else if (fontName.len())
+			*(m_ps->m_fontName) = fontName;
 	}
 }
 
@@ -788,7 +794,7 @@ void WP6ContentListener::attributeChange(const bool isOn, const uint8_t attribut
 	}
 }
 
-void WP6ContentListener::spacingAfterParagraphChange(const float spacingRelative, const float spacingAbsolute)
+void WP6ContentListener::spacingAfterParagraphChange(const double spacingRelative, const double spacingAbsolute)
 {
 	if (!isUndoOn())
 	{
@@ -798,7 +804,7 @@ void WP6ContentListener::spacingAfterParagraphChange(const float spacingRelative
 		// We have to recompute this every change of fontSize. That is why we keep the two components in
 		// m_parsingState and the following formula is to be found in the fontChange(...) function as well.
 		m_ps->m_paragraphMarginBottom =
-			(float)(((m_parseState->m_paragraphMarginBottomRelative - 1.0f)*m_ps->m_fontSize)/72.0f) +
+			(double)(((m_parseState->m_paragraphMarginBottomRelative - 1.0)*m_ps->m_fontSize)/72.0) +
 			m_parseState->m_paragraphMarginBottomAbsolute;
 		// Variable spacingAfterParagraphRelative already contains the height of the space in inches
 	}
@@ -808,20 +814,20 @@ void WP6ContentListener::marginChange(uint8_t side, uint16_t margin)
 {
 	if (!isUndoOn())
 	{
-		float marginInch = (float)((double)margin/ (double)WPX_NUM_WPUS_PER_INCH);
+		double marginInch = (double)((double)margin/ (double)WPX_NUM_WPUS_PER_INCH);
 
 		switch(side)
 		{
 		case WPX_LEFT:
 			if (m_ps->m_numColumns > 1)
 			{
-				m_ps->m_leftMarginByPageMarginChange = 0.0f;
+				m_ps->m_leftMarginByPageMarginChange = 0.0;
 				m_ps->m_sectionMarginLeft = marginInch - m_ps->m_pageMarginLeft;
 			}
 			else
 			{
 				m_ps->m_leftMarginByPageMarginChange = marginInch - m_ps->m_pageMarginLeft;
-				m_ps->m_sectionMarginLeft = 0.0f;
+				m_ps->m_sectionMarginLeft = 0.0;
 			}
 			m_ps->m_paragraphMarginLeft = m_ps->m_leftMarginByPageMarginChange
 						+ m_ps->m_leftMarginByParagraphMarginChange
@@ -830,13 +836,13 @@ void WP6ContentListener::marginChange(uint8_t side, uint16_t margin)
 		case WPX_RIGHT:
 			if (m_ps->m_numColumns > 1)
 			{
-				m_ps->m_rightMarginByPageMarginChange = 0.0f;
+				m_ps->m_rightMarginByPageMarginChange = 0.0;
 				m_ps->m_sectionMarginRight = marginInch - m_ps->m_pageMarginRight;
 			}
 			else
 			{
 				m_ps->m_rightMarginByPageMarginChange = marginInch - m_ps->m_pageMarginRight;
-				m_ps->m_sectionMarginRight = 0.0f;
+				m_ps->m_sectionMarginRight = 0.0;
 			}
 			m_ps->m_paragraphMarginRight = m_ps->m_rightMarginByPageMarginChange
 						+ m_ps->m_rightMarginByParagraphMarginChange
@@ -863,7 +869,7 @@ void WP6ContentListener::paragraphMarginChange(uint8_t side, int16_t margin)
 
 		m_ps->m_currentListLevel = 0;
 
-		float marginInch = (float)((double)margin / (double)WPX_NUM_WPUS_PER_INCH);
+		double marginInch = (double)((double)margin / (double)WPX_NUM_WPUS_PER_INCH);
 		switch(side)
 		{
 		case WPX_LEFT:
@@ -895,7 +901,7 @@ void WP6ContentListener::indentFirstLineChange(int16_t offset)
 {
 	if (!isUndoOn())
 	{
-		float offsetInch = (float)((double)offset / (double)WPX_NUM_WPUS_PER_INCH);
+		double offsetInch = (double)((double)offset / (double)WPX_NUM_WPUS_PER_INCH);
 		m_ps->m_textIndentByParagraphIndentChange = offsetInch;
 		// This is necessary in case we have Indent First Line and Hard Back Tab
 		// in the same time. The Hard Back Tab applies to the current paragraph
@@ -909,7 +915,7 @@ void WP6ContentListener::indentFirstLineChange(int16_t offset)
 }
 
 void WP6ContentListener::columnChange(const WPXTextColumnType /* columnType */, const uint8_t numColumns,
-					const std::vector<float> &columnWidth, const std::vector<bool> &isFixedWidth)
+					const std::vector<double> &columnWidth, const std::vector<bool> &isFixedWidth)
 {
 	if (!isUndoOn())
 	{
@@ -922,7 +928,7 @@ void WP6ContentListener::columnChange(const WPXTextColumnType /* columnType */,
 		m_ps->m_isParagraphColumnBreak = false;
 		m_ps->m_isTextColumnWithoutParagraph = false;
 
-		float remainingSpace = m_ps->m_pageFormWidth - m_ps->m_pageMarginLeft - m_ps->m_sectionMarginLeft
+		double remainingSpace = m_ps->m_pageFormWidth - m_ps->m_pageMarginLeft - m_ps->m_sectionMarginLeft
 			- m_ps->m_pageMarginRight - m_ps->m_sectionMarginRight
 			- m_ps->m_leftMarginByPageMarginChange - m_ps->m_rightMarginByPageMarginChange;
 		// determine the space that is to be divided between columns whose width is expressed in percentage of remaining space
@@ -940,14 +946,14 @@ void WP6ContentListener::columnChange(const WPXTextColumnType /* columnType */,
 			for (i=0; i<numColumns; i++)
 			{
 				if (i == 0)
-					tmpColumn.m_leftGutter = 0.0f;
+					tmpColumn.m_leftGutter = 0.0;
 				else if (isFixedWidth[2*i-1])
 					tmpColumn.m_leftGutter = 0.5f * columnWidth[2*i-1];
 				else
 					tmpColumn.m_leftGutter = 0.5f * remainingSpace * columnWidth[2*i-1];
 				
 				if (i >= (numColumns - 1))
-					tmpColumn.m_rightGutter = 0.0f;
+					tmpColumn.m_rightGutter = 0.0;
 				else if (isFixedWidth[2*i+1])
 					tmpColumn.m_rightGutter = 0.5f * columnWidth[2*i+1];
 				else
@@ -1032,8 +1038,8 @@ void WP6ContentListener::displayNumberReferenceGroupOn(const uint8_t subGroup, c
 	{
 		switch (subGroup)
 		{
-		case WP6_DISPLAY_NUMBER_REFERENCE_GROUP_PARAGRAPH_NUMBER_ON:
-		case WP6_DISPLAY_NUMBER_REFERENCE_GROUP_USER_DEFINED_ON:
+		case WP6_DISPLAY_NUMBER_REFERENCE_GROUP_PARAGRAPH_NUMBER_DISPLAY_ON:
+		case WP6_DISPLAY_NUMBER_REFERENCE_GROUP_USER_DEFINED_NUMBER_DISPLAY_ON:
 			// HACKISH: if we are in a paragraph style sequence we will pretend that paragraph numbering was
 			// just turned on even though it didn't happen
 			if (m_parseState->m_styleStateSequence.getCurrentState() == BEGIN_BEFORE_NUMBERING)
@@ -1051,8 +1057,12 @@ void WP6ContentListener::displayNumberReferenceGroupOn(const uint8_t subGroup, c
 			}
 			m_parseState->m_putativeListElementHasDisplayReferenceNumber = true;
 			break;
-		case WP6_DISPLAY_NUMBER_REFERENCE_GROUP_FOOTNOTE_NUMBER_ON:
-		case WP6_DISPLAY_NUMBER_REFERENCE_GROUP_ENDNOTE_NUMBER_ON:
+		case WP6_DISPLAY_NUMBER_REFERENCE_GROUP_FOOTNOTE_NUMBER_DISPLAY_ON:
+		case WP6_DISPLAY_NUMBER_REFERENCE_GROUP_ENDNOTE_NUMBER_DISPLAY_ON:
+			m_parseState->m_styleStateSequence.setCurrentState(DISPLAY_REFERENCING);
+			break;
+		case WP6_DISPLAY_NUMBER_REFERENCE_GROUP_PAGE_NUMBER_DISPLAY_ON:
+		case WP6_DISPLAY_NUMBER_REFERENCE_GROUP_TOTAL_NUMBER_OF_PAGES_DISPLAY_ON:
 			m_parseState->m_styleStateSequence.setCurrentState(DISPLAY_REFERENCING);
 			break;
 		}
@@ -1065,8 +1075,8 @@ void WP6ContentListener::displayNumberReferenceGroupOff(const uint8_t subGroup)
 	{
 		switch (subGroup)
 		{
-		case WP6_DISPLAY_NUMBER_REFERENCE_GROUP_PARAGRAPH_NUMBER_OFF:
-		case WP6_DISPLAY_NUMBER_REFERENCE_GROUP_USER_DEFINED_OFF:
+		case WP6_DISPLAY_NUMBER_REFERENCE_GROUP_PARAGRAPH_NUMBER_DISPLAY_OFF:
+		case WP6_DISPLAY_NUMBER_REFERENCE_GROUP_USER_DEFINED_NUMBER_DISPLAY_OFF:
 			if (m_parseState->m_styleStateSequence.getPreviousState() == BEGIN_NUMBERING_BEFORE_DISPLAY_REFERENCING)
 				m_parseState->m_styleStateSequence.setCurrentState(BEGIN_NUMBERING_AFTER_DISPLAY_REFERENCING);
 			else {
@@ -1080,8 +1090,28 @@ void WP6ContentListener::displayNumberReferenceGroupOff(const uint8_t subGroup)
 
 			}
 			break;
-		case WP6_DISPLAY_NUMBER_REFERENCE_GROUP_FOOTNOTE_NUMBER_OFF:
-		case WP6_DISPLAY_NUMBER_REFERENCE_GROUP_ENDNOTE_NUMBER_OFF:
+		case WP6_DISPLAY_NUMBER_REFERENCE_GROUP_FOOTNOTE_NUMBER_DISPLAY_OFF:
+		case WP6_DISPLAY_NUMBER_REFERENCE_GROUP_ENDNOTE_NUMBER_DISPLAY_OFF:
+			m_parseState->m_styleStateSequence.setCurrentState(m_parseState->m_styleStateSequence.getPreviousState());
+			break;
+		case WP6_DISPLAY_NUMBER_REFERENCE_GROUP_PAGE_NUMBER_DISPLAY_OFF:
+		case WP6_DISPLAY_NUMBER_REFERENCE_GROUP_TOTAL_NUMBER_OF_PAGES_DISPLAY_OFF:
+			m_parseState->m_numberText.clear();
+			_flushText();
+			_openSpan();
+			// in theory the page numbering type should only apply to thepage number itself, however, I can't think of a case where you'd
+			// want the total num of pages to be in a different format plus I don't see a way of changing that. so...
+			WPXPropertyList propList;
+			propList.insert("style:num-format", _numberingTypeToString(m_parseState->m_currentPageNumberingType));
+
+			if (subGroup == WP6_DISPLAY_NUMBER_REFERENCE_GROUP_PAGE_NUMBER_DISPLAY_OFF) 
+			{
+				m_documentInterface->insertField(WPXString("text:page-number"), propList);
+			}
+			else // WP6_DISPLAY_NUMBER_REFERENCE_GROUP_TOTAL_NUMBER_OF_PAGES_DISPLAY_OFF
+			{
+				m_documentInterface->insertField(WPXString("text:page-count"), propList);
+			}
 			m_parseState->m_styleStateSequence.setCurrentState(m_parseState->m_styleStateSequence.getPreviousState());
 			break;
 		}
@@ -1206,23 +1236,31 @@ void WP6ContentListener::noteOff(const WPXNoteType noteType)
 			propList.insert("libwpd:number", number);
 
 		if (noteType == FOOTNOTE)
-			m_listenerImpl->openFootnote(propList);
+			m_documentInterface->openFootnote(propList);
 		else
-			m_listenerImpl->openEndnote(propList);
+			m_documentInterface->openEndnote(propList);
 
 		uint16_t textPID = (uint16_t)m_parseState->m_noteTextPID;
 		handleSubDocument(((textPID && WP6Listener::getPrefixDataPacket(textPID)) ? WP6Listener::getPrefixDataPacket(textPID)->getSubDocument() : 0), 
-				false, m_parseState->m_tableList, m_parseState->m_nextTableIndice);
+				WPX_SUBDOCUMENT_NOTE, m_parseState->m_tableList, m_parseState->m_nextTableIndice);
 
 		if (noteType == FOOTNOTE)
-			m_listenerImpl->closeFootnote();
+			m_documentInterface->closeFootnote();
 		else
-			m_listenerImpl->closeEndnote();
+			m_documentInterface->closeEndnote();
 		m_ps->m_isNote = false;
 		m_parseState->m_numNestedNotes = 0;
 	}
 }
 
+void WP6ContentListener::setPageNumberingType(const WPXNumberingType pageNumberingType)
+{
+	if (!isUndoOn()) 
+	{
+		m_parseState->m_currentPageNumberingType = pageNumberingType;
+	}
+}
+
 void WP6ContentListener::defineTable(const uint8_t position, const uint16_t leftOffset)
 {
 	if (!isUndoOn())
@@ -1258,7 +1296,7 @@ void WP6ContentListener::defineTable(const uint8_t position, const uint16_t left
 			break;
 		}
 		// Note: WordPerfect has an offset from the left edge of the page. We translate it to the offset from the left margin
-		m_ps->m_tableDefinition.m_leftOffset = (float)((double)leftOffset / (double)WPX_NUM_WPUS_PER_INCH) - m_ps->m_paragraphMarginLeft;
+		m_ps->m_tableDefinition.m_leftOffset = (double)((double)leftOffset / (double)WPX_NUM_WPUS_PER_INCH) - m_ps->m_paragraphMarginLeft;
 
 		// remove all the old column information
 		m_ps->m_tableDefinition.m_columns.clear();
@@ -1280,9 +1318,9 @@ void WP6ContentListener::addTableColumnDefinition(const uint32_t width, const ui
 	{
 		// define the new column
 		WPXColumnDefinition colDef;
-		colDef.m_width = (float)((double)width / (double)WPX_NUM_WPUS_PER_INCH);
-		colDef.m_leftGutter = (float)((double)width / (double)WPX_NUM_WPUS_PER_INCH);
-		colDef.m_rightGutter = (float)((double)width / (double)WPX_NUM_WPUS_PER_INCH);
+		colDef.m_width = (double)((double)width / (double)WPX_NUM_WPUS_PER_INCH);
+		colDef.m_leftGutter = (double)((double)width / (double)WPX_NUM_WPUS_PER_INCH);
+		colDef.m_rightGutter = (double)((double)width / (double)WPX_NUM_WPUS_PER_INCH);
 
 		WPXColumnProperties colProp;
 		colProp.m_attributes = attributes;
@@ -1319,7 +1357,7 @@ void WP6ContentListener::startTable()
 		if (!m_parseState->m_currentTable)
 			throw ParseException();
 		
-		if (!m_parseState->m_currentTable->isEmpty()) // WordPerfect ignores empty tables and inserts an Hard EOL instead
+		if (!m_parseState->m_currentTable->isEmpty())
 			_openTable();
 			
 	}
@@ -1330,7 +1368,7 @@ void WP6ContentListener::insertRow(const uint16_t rowHeight, const bool isMinimu
 	if (!isUndoOn() && m_ps->m_isTableOpened)
 	{
 		_flushText();
-		float rowHeightInch = (float)((double) rowHeight / (double)WPX_NUM_WPUS_PER_INCH);
+		double rowHeightInch = (double)((double) rowHeight / (double)WPX_NUM_WPUS_PER_INCH);
 		_openTableRow(rowHeightInch, isMinimumHeight, isHeaderRow);
 	}
 }
@@ -1372,11 +1410,14 @@ void WP6ContentListener::insertCell(const uint8_t colSpan, const uint8_t rowSpan
 			       	m_ps->m_currentTableCellNumberInRow)->m_borderBits, cellFgColor, cellBgColor,      
 			       cellBorderColor, cellVerticalAlignment);
 
+		m_ps->m_cellAttributeBits = 0;
 		if (useCellAttributes)
 			m_ps->m_cellAttributeBits = cellAttributes;
-		else
+		else if (m_ps->m_currentTableCol > 0 && m_ps->m_tableDefinition.m_columnsProperties.size() >= (unsigned)m_ps->m_currentTableCol)
 			m_ps->m_cellAttributeBits = m_ps->m_tableDefinition.m_columnsProperties[m_ps->m_currentTableCol-1].m_attributes;
-		justificationChange(m_ps->m_tableDefinition.m_columnsProperties[m_ps->m_currentTableCol-1].m_alignment);
+
+		if (m_ps->m_currentTableCol > 0 && m_ps->m_tableDefinition.m_columnsProperties.size() >= (unsigned)m_ps->m_currentTableCol)
+			justificationChange(m_ps->m_tableDefinition.m_columnsProperties[m_ps->m_currentTableCol-1].m_alignment);
 	}
 }
 
@@ -1392,7 +1433,305 @@ void WP6ContentListener::endTable()
 	}
 }
 
-void WP6ContentListener::_handleSubDocument(const WPXSubDocument *subDocument, const bool isHeaderFooter, WPXTableList tableList, int nextTableIndice)
+void WP6ContentListener::boxOn(const uint8_t /* anchoringType */, const uint8_t generalPositioningFlags, const uint8_t horizontalPositioningFlags,
+		const int16_t horizontalOffset, const uint8_t /* leftColumn */, const uint8_t /* rightColumn */, const uint8_t verticalPositioningFlags,
+		const int16_t verticalOffset, const uint8_t widthFlags, const uint16_t width, const uint8_t heightFlags, const uint16_t height,
+		const uint8_t boxContentType, const uint16_t nativeWidth, const uint16_t nativeHeight)
+{
+	if (isUndoOn() || (m_ps->m_isTableOpened && !m_ps->m_isTableCellOpened))
+		return;
+		 
+	if (!m_ps->m_isSpanOpened)
+		_openSpan();
+	else
+		_flushText();
+
+	WPXPropertyList propList;
+
+	if (heightFlags & 0x01)
+		propList.insert("style:rel-height", "scale");
+	else
+		propList.insert("svg:height", (double)((double)height/(double)WPX_NUM_WPUS_PER_INCH));
+
+	if (widthFlags & 0x01)
+		propList.insert("style:rel-width", "scale");
+	else
+		propList.insert("svg:width", (double)((double)width/(double)WPX_NUM_WPUS_PER_INCH));
+
+	if (boxContentType != 0x01) // This seems to work for the text boxes only
+	{
+		propList.insert("svg:height", (double)((double)height/(double)WPX_NUM_WPUS_PER_INCH));
+		propList.insert("svg:width", (double)((double)width/(double)WPX_NUM_WPUS_PER_INCH));
+	}
+		
+	if ((boxContentType == 0x03) && nativeWidth && nativeHeight)
+	{
+		
+		if ((heightFlags & 0x01) && (widthFlags & 0x01))
+		{
+			propList.insert("svg:height", (double)((double)nativeHeight/(double)WPX_NUM_WPUS_PER_INCH));
+			propList.insert("svg:width", (double)((double)nativeWidth/(double)WPX_NUM_WPUS_PER_INCH));
+		}
+		else
+		{
+			if (heightFlags & 0x01)
+				propList.insert("svg:height", (double)((double)width * (double)nativeHeight /
+					((double)nativeWidth * (double)WPX_NUM_WPUS_PER_INCH)));
+			if (widthFlags & 0x01)
+				propList.insert("svg:width", (double)((double)height * (double)nativeWidth /
+					((double)nativeHeight * (double)WPX_NUM_WPUS_PER_INCH)));
+		}
+	} 
+
+	if (horizontalOffset)
+		propList.insert("svg:x", (double)((double)horizontalOffset/(double)WPX_NUM_WPUS_PER_INCH));
+
+	switch (generalPositioningFlags & 0x03)
+	{
+	case 0x00: // Page anchored
+		propList.insert("text:anchor-type", "char"); // ugly workaround to remediate OOo's wrong implementation of ODF
+		switch (horizontalPositioningFlags & 0x03)
+		{
+		case 0x00:
+			propList.insert("style:horizontal-rel", "page");
+			if (horizontalOffset)
+				propList.insert("style:horizontal-pos", "from-left");
+			else
+				propList.insert("style:horizontal-pos", "left");
+			break;
+		case 0x01:
+		case 0x02:
+			switch ((horizontalPositioningFlags & 0x1C) >> 2)
+			{
+			case 0x00:
+				propList.insert("style:horizontal-rel", "page-content");
+				propList.insert("style:horizontal-pos", "from-left");
+				propList.insert("svg:x", (double)((double)horizontalOffset/(double)WPX_NUM_WPUS_PER_INCH)
+					+ m_ps->m_leftMarginByPageMarginChange + m_ps->m_sectionMarginLeft);
+				break;
+			case 0x01:
+				propList.insert("style:horizontal-rel", "page-end-margin");
+				propList.insert("style:horizontal-pos", "from-left");
+				propList.insert("svg:x", (double)((double)horizontalOffset/(double)WPX_NUM_WPUS_PER_INCH)
+					- (double)((double)width/(double)WPX_NUM_WPUS_PER_INCH)
+					- m_ps->m_rightMarginByPageMarginChange - m_ps->m_sectionMarginRight);
+				break;
+			case 0x02:
+				propList.insert("style:horizontal-rel", "page-content");
+				propList.insert("style:horizontal-pos", "center");
+				break;
+			case 0x03:
+				break;
+			default:
+				break;
+			}
+			break;
+		default:
+			break;
+		}
+		break;
+	case 0x01: // Paragraph anchored
+		propList.insert("text:anchor-type", "paragraph");
+		switch (horizontalPositioningFlags & 0x03)
+		{
+		case 0x00:
+			propList.insert("style:horizontal-rel", "page");
+			if (horizontalOffset)
+				propList.insert("style:horizontal-pos", "from-left");
+			else
+				propList.insert("style:horizontal-pos", "left");
+			break;
+		case 0x01:
+		case 0x02:
+			switch ((horizontalPositioningFlags & 0x1C) >> 2)
+			{
+			case 0x00:
+				propList.insert("style:horizontal-rel", "page-content");
+				propList.insert("style:horizontal-pos", "from-left");
+				propList.insert("svg:x", (double)((double)horizontalOffset/(double)WPX_NUM_WPUS_PER_INCH)
+					+ m_ps->m_leftMarginByPageMarginChange + m_ps->m_sectionMarginLeft);
+				break;
+			case 0x01:
+				propList.insert("style:horizontal-rel", "page-end-margin");
+				propList.insert("style:horizontal-pos", "from-left");
+				propList.insert("svg:x", (double)((double)horizontalOffset/(double)WPX_NUM_WPUS_PER_INCH)
+					- (double)((double)width/(double)WPX_NUM_WPUS_PER_INCH)
+					- m_ps->m_rightMarginByPageMarginChange - m_ps->m_sectionMarginRight);
+				break;
+			case 0x02:
+				propList.insert("style:horizontal-rel", "page-content");
+				propList.insert("style:horizontal-pos", "center");
+				break;
+			case 0x03:
+				break;
+			default:
+				break;
+			}
+			break;
+		default:
+			break;
+		}
+		break;
+	case 0x02: // Character anchored
+		propList.insert("text:anchor-type", "as-char");
+		break;
+	default:
+		break;
+	}
+		
+	switch (horizontalPositioningFlags & 0x03)
+	{
+	case 0x00:
+		propList.insert("style:horizontal-rel", "page");
+		if (horizontalOffset)
+			propList.insert("style:horizontal-pos", "from-left");
+		else
+			propList.insert("style:horizontal-pos", "left");
+		break;
+	case 0x01:
+	case 0x02:
+		switch ((horizontalPositioningFlags & 0x1C) >> 2)
+		{
+		case 0x00:
+			propList.insert("style:horizontal-rel", "page-content");
+			propList.insert("style:horizontal-pos", "from-left");
+			propList.insert("svg:x", (double)((double)horizontalOffset/(double)WPX_NUM_WPUS_PER_INCH)
+				+ m_ps->m_leftMarginByPageMarginChange + m_ps->m_sectionMarginLeft);
+			break;
+		case 0x01:
+			propList.insert("style:horizontal-rel", "page-end-margin");
+			propList.insert("style:horizontal-pos", "from-left");
+			propList.insert("svg:x", (double)((double)horizontalOffset/(double)WPX_NUM_WPUS_PER_INCH)
+				- (double)((double)width/(double)WPX_NUM_WPUS_PER_INCH)
+				- m_ps->m_rightMarginByPageMarginChange - m_ps->m_sectionMarginRight);
+			break;
+		case 0x02:
+			propList.insert("style:horizontal-rel", "page-content");
+			propList.insert("style:horizontal-pos", "center");
+			break;
+		case 0x03:
+			break;
+		default:
+			break;
+		}
+		break;
+	default:
+		break;
+	}
+	
+	if (verticalOffset)
+		propList.insert("svg:y", (double)((double)verticalOffset/(double)WPX_NUM_WPUS_PER_INCH));
+	
+	switch (verticalPositioningFlags & 0x03)
+	{
+	case 0x00:
+		propList.insert("style:vertical-rel", "page");
+		propList.insert("style:vertical-pos", "from-top");
+		// we have to remediate the workaround with alignment
+		// once fixed, remove this hack !!!
+		propList.insert("svg:y", (double)(((double)verticalOffset/(double)WPX_NUM_WPUS_PER_INCH) - m_ps->m_pageMarginTop));
+		break;
+	case 0x01:
+	    if ((generalPositioningFlags & 0x03) == 0x00)
+			propList.insert("style:vertical-rel", "page-content");
+		else if ((generalPositioningFlags & 0x03) == 0x01)
+			propList.insert("style:vertical-rel", "paragraph");
+		else if ((generalPositioningFlags & 0x03) == 0x02)
+			propList.insert("style:vertical-rel", "baseline");
+		switch((verticalPositioningFlags & 0x1C) >> 2)
+		{
+		case 0x00:
+		case 0x03:
+			propList.insert("style:vertical-pos", "from-top");
+			break;
+		case 0x01:
+			propList.insert("style:vertical-pos", "bottom");
+			break;
+		case 0x02:
+			propList.insert("style:vertical-pos", "middle");
+			break;
+		default:
+			break;
+		}
+		break;
+	default:
+		break;
+	}
+	
+/*	if (propList["text:anchor-type"] && propList["text:anchor-type"]->getStr() == "page")
+		propList.insert("text:anchor-page-number", m_ps->m_currentPageNumber); */
+
+	m_documentInterface->openFrame(propList);
+	m_parseState->m_isFrameOpened = true;
+}
+
+void WP6ContentListener::boxOff()
+{
+	if (!isUndoOn() && m_parseState->m_isFrameOpened)
+	{
+		m_documentInterface->closeFrame();
+		m_parseState->m_isFrameOpened = false;
+	}
+}
+
+void WP6ContentListener::insertGraphicsData(const uint16_t packetId)
+{
+	if (isUndoOn() || !m_parseState->m_isFrameOpened)
+		return;
+
+	if (const WP6GraphicsCachedFileDataPacket *gcfdPacket = dynamic_cast<const WP6GraphicsCachedFileDataPacket *>(this->getPrefixDataPacket(packetId))) 
+	{
+		WPXPropertyList propList;
+		propList.insert("libwpd:mimetype", "image/x-wpg");
+		if (gcfdPacket->getBinaryObject())
+			m_documentInterface->insertBinaryObject(propList, *(gcfdPacket->getBinaryObject()));
+	}
+}
+
+void WP6ContentListener::insertTextBox(const WP6SubDocument *subDocument)
+{
+	if (!isUndoOn() && subDocument && m_parseState->m_isFrameOpened)
+	{
+		WPXPropertyList propList;
+		m_documentInterface->openTextBox(propList);
+		
+		// Positioned objects like text boxes are special beasts. They can contain all hierarchical elements up
+		// to the level of sections. They cannot open or close a page span though.
+		handleSubDocument(subDocument, WPX_SUBDOCUMENT_TEXT_BOX, m_parseState->m_tableList, m_parseState->m_nextTableIndice);
+
+		m_documentInterface->closeTextBox();
+	}
+}
+	
+
+void WP6ContentListener::commentAnnotation(const uint16_t textPID)
+{
+	if (!isUndoOn())
+	{
+		if (!m_ps->m_isParagraphOpened)
+			_openParagraph();
+		else
+		{
+			_flushText();
+			_closeSpan();
+		}
+
+		WPXPropertyList propList;
+		m_documentInterface->openComment(propList);
+		
+		m_ps->m_isNote = true;
+
+		handleSubDocument(((textPID && WP6Listener::getPrefixDataPacket(textPID)) ? WP6Listener::getPrefixDataPacket(textPID)->getSubDocument() : 0), 
+				WPX_SUBDOCUMENT_COMMENT_ANNOTATION, m_parseState->m_tableList, m_parseState->m_nextTableIndice);
+
+		m_ps->m_isNote = false;
+
+		m_documentInterface->closeComment();
+	}
+}
+
+void WP6ContentListener::_handleSubDocument(const WPXSubDocument *subDocument, WPXSubDocumentType subDocumentType, 
+	WPXTableList tableList, int nextTableIndice)
 {
 	// save our old parsing state on our "stack"
 	WP6ContentParsingState *oldParseState = m_parseState;
@@ -1400,7 +1739,7 @@ void WP6ContentListener::_handleSubDocument(const WPXSubDocument *subDocument, c
 	m_parseState = new WP6ContentParsingState(tableList, nextTableIndice);
 	m_parseState->m_numNestedNotes = oldParseState->m_numNestedNotes;
 
-	if (isHeaderFooter)
+	if (subDocumentType == WPX_SUBDOCUMENT_HEADER_FOOTER)
 	{
 		// is it is Header or Footer, assume that the initial page margins are of 1 inch.
 		// This is a behaviour that I observed with WP10 -- Fridrich
@@ -1429,7 +1768,7 @@ void WP6ContentListener::_handleSubDocument(const WPXSubDocument *subDocument, c
 #endif
 
 	// if we are in a foot/endNote, the nextTableIndice sequence has to be maintained
-	if (!isHeaderFooter)
+	if (subDocumentType != WPX_SUBDOCUMENT_HEADER_FOOTER)
 		oldParseState->m_nextTableIndice = m_parseState->m_nextTableIndice;
 	oldParseState->m_numNestedNotes = m_parseState->m_numNestedNotes;
 
@@ -1464,44 +1803,44 @@ void WP6ContentListener::_flushText()
 	
 	if (m_parseState->m_textBeforeNumber.len())
 	{
-		m_listenerImpl->insertText(m_parseState->m_textBeforeNumber);
+		_insertText(m_parseState->m_textBeforeNumber);
 		m_parseState->m_textBeforeNumber.clear();
 	}
 	
 	if (m_parseState->m_textBeforeDisplayReference.len())
 	{
-		m_listenerImpl->insertText(m_parseState->m_textBeforeDisplayReference);
+            	_insertText(m_parseState->m_textBeforeDisplayReference);
 		m_parseState->m_textBeforeDisplayReference.clear();
 	}
 	
 	if (m_parseState->m_numberText.len())
 	{
-		m_listenerImpl->insertText(m_parseState->m_numberText);
+		_insertText(m_parseState->m_numberText);
 		m_parseState->m_numberText.clear();
 	}
 	
 	if (m_parseState->m_textAfterDisplayReference.len())
 	{
-		m_listenerImpl->insertText(m_parseState->m_textAfterDisplayReference);
+		_insertText(m_parseState->m_textAfterDisplayReference);
 		m_parseState->m_textAfterDisplayReference.clear();
 	}
 	
 	if (m_parseState->m_textAfterNumber.len())
 	{
-		m_listenerImpl->insertText(m_parseState->m_textAfterNumber);
+		_insertText(m_parseState->m_textAfterNumber);
 		m_parseState->m_textAfterNumber.clear();
 	}
 	
 	if (m_parseState->m_numListExtraTabs > 0)
 	{
 		for (; m_parseState->m_numListExtraTabs > 0; m_parseState->m_numListExtraTabs--)
-			m_listenerImpl->insertTab();
+			m_documentInterface->insertTab();
 		m_parseState->m_numListExtraTabs = 0;
 	}
 
 	if (m_parseState->m_bodyText.len())
 	{
-		m_listenerImpl->insertText(m_parseState->m_bodyText);
+		_insertText(m_parseState->m_bodyText);
 		m_parseState->m_bodyText.clear();
 	}
 	
@@ -1553,7 +1892,7 @@ void WP6ContentListener::_handleListChange(const uint16_t outlineHash)
 			propList.insert("text:min-label-width", m_ps->m_paragraphMarginLeft + m_ps->m_paragraphTextIndent - m_ps->m_listReferencePosition);
 			propList.insert("text:space-before", m_ps->m_listReferencePosition - m_ps->m_listBeginPosition);
 			
-			m_listenerImpl->defineOrderedListLevel(propList);
+			m_documentInterface->defineOrderedListLevel(propList);
 		}
 		else
 		{
@@ -1561,7 +1900,7 @@ void WP6ContentListener::_handleListChange(const uint16_t outlineHash)
 			propList.insert("text:min-label-width", m_ps->m_paragraphMarginLeft + m_ps->m_paragraphTextIndent - m_ps->m_listReferencePosition);
 			propList.insert("text:space-before", m_ps->m_listReferencePosition - m_ps->m_listBeginPosition);
 			
-			m_listenerImpl->defineUnorderedListLevel(propList);
+			m_documentInterface->defineUnorderedListLevel(propList);
 		}
 		for (int i=(oldListLevel+1); i<=m_ps->m_currentListLevel; i++)
 		{
@@ -1574,12 +1913,12 @@ void WP6ContentListener::_handleListChange(const uint16_t outlineHash)
 
 			if (m_parseState->m_putativeListElementHasDisplayReferenceNumber)
 			{
-				m_listenerImpl->openOrderedListLevel(propList2);
+				m_documentInterface->openOrderedListLevel(propList2);
 				m_parseState->m_listTypeStack.push(ORDERED);
 			}
 			else
 			{
-				m_listenerImpl->openUnorderedListLevel(propList2);
+				m_documentInterface->openUnorderedListLevel(propList2);
 				m_parseState->m_listTypeStack.push(UNORDERED);
 			}
 		}
@@ -1601,9 +1940,9 @@ void WP6ContentListener::_handleListChange(const uint16_t outlineHash)
 			// a priori and I hate writing lame excuses like this, so we might want to
 			// change this at some point
 			if (tmpListType == UNORDERED)
-				m_listenerImpl->closeUnorderedListLevel();
+				m_documentInterface->closeUnorderedListLevel();
 			else
-				m_listenerImpl->closeOrderedListLevel();
+				m_documentInterface->closeOrderedListLevel();
 		}
 	}
 
