@@ -35,7 +35,7 @@
 #include <map>
 #include <vector>
 
-class WPXHLListenerImpl;
+class WPXDocumentInterface;
 class WPXTable;
 
 enum WP6StyleState { NORMAL, DOCUMENT_NOTE, DOCUMENT_NOTE_GLOBAL,
@@ -53,8 +53,8 @@ class WP6StyleStateSequence
 public:
 	WP6StyleStateSequence() : m_stateSequence(), m_currentState(), m_previousState() { clear(); }
 	void setCurrentState(WP6StyleState state) { for (int i=(STATE_MEMORY-1); i>0; i--) m_stateSequence[i] = m_stateSequence[i-1]; m_stateSequence[0]=state; }
-	const WP6StyleState getCurrentState() const { return m_stateSequence[0]; /*currentState;*/ }
-	const WP6StyleState getPreviousState() const { return m_stateSequence[1]; /*m_previousState;*/ }
+	WP6StyleState getCurrentState() const { return m_stateSequence[0]; /*currentState;*/ }
+	WP6StyleState getPreviousState() const { return m_stateSequence[1]; /*m_previousState;*/ }
 	void clear() { m_stateSequence.clear(); for (int i=0; i<STATE_MEMORY; i++) m_stateSequence.push_back(NORMAL); }
 
 private:
@@ -75,8 +75,8 @@ struct _WP6ContentParsingState
 	WPXString m_textAfterDisplayReference;
 	WPXString m_textAfterNumber;
 
-	float m_paragraphMarginBottomRelative;
-	float m_paragraphMarginBottomAbsolute;
+	double m_paragraphMarginBottomRelative;
+	double m_paragraphMarginBottomAbsolute;
 
 	int m_numRemovedParagraphBreaks;
 	
@@ -97,11 +97,16 @@ struct _WP6ContentParsingState
 
 	int m_noteTextPID;
 	int m_numNestedNotes;
+	
+	bool m_isFrameOpened;
 
 	uint16_t m_leaderCharacter;
 	uint8_t m_leaderNumSpaces;
 	std::vector<WPXTabStop> m_tempTabStops;
 	std::vector<bool> m_tempUsePreWP9LeaderMethod;
+
+	WPXNumberingType m_currentPageNumberingType;
+
 private:
 	_WP6ContentParsingState(const _WP6ContentParsingState&);
 	_WP6ContentParsingState& operator=(const _WP6ContentParsingState&);
@@ -133,11 +138,12 @@ private:
 class WP6ContentListener : public WP6Listener, protected WPXContentListener
 {
 public:
-	WP6ContentListener(std::list<WPXPageSpan> &pageList, WPXTableList tableList, WPXHLListenerImpl *listenerImpl);
+	WP6ContentListener(std::list<WPXPageSpan> &pageList, WPXTableList tableList, WPXDocumentInterface *documentInterface);
 	~WP6ContentListener();
 
 	void startDocument() { WPXContentListener::startDocument(); };
-        void setDate(const uint16_t type, const uint16_t year, 
+	void startSubDocument() { WPXContentListener::startSubDocument(); };
+	void setDate(const uint16_t type, const uint16_t year, 
 		     const uint8_t month, const uint8_t day, 
 		     const uint8_t hour, const uint8_t minute, 
 		     const uint8_t second, const uint8_t dayOfWeek, 
@@ -148,24 +154,25 @@ public:
 	void defineTabStops(const bool isRelative, const std::vector<WPXTabStop> &tabStops, 
 				    const std::vector<bool> &usePreWP9LeaderMethods);
 	void insertCharacter(const uint16_t character);
-	void insertTab(const uint8_t tabType, float tabPosition);
+	void insertTab(const uint8_t tabType, double tabPosition);
 	void handleLineBreak();
 	void insertEOL();
 	void insertBreak(const uint8_t breakType) { WPXContentListener::insertBreak(breakType); };
-	void lineSpacingChange(const float lineSpacing) { WPXContentListener::lineSpacingChange(lineSpacing); };
+	void lineSpacingChange(const double lineSpacing) { WPXContentListener::lineSpacingChange(lineSpacing); };
 	void justificationChange(const uint8_t justification) { WPXContentListener::justificationChange(justification); };
 	void characterColorChange(const uint8_t red, const uint8_t green, const uint8_t blue);
 	void characterShadingChange(const uint8_t shading);
 	void highlightChange(const bool isOn, const RGBSColor color);
-	void fontChange(const uint16_t matchedFontPointSize, const uint16_t fontPID);
+	void fontChange(const uint16_t matchedFontPointSize, const uint16_t fontPID, const WPXString &fontName);
  	void attributeChange(const bool isOn, const uint8_t attribute);
-	void spacingAfterParagraphChange(const float spacingRelative, const float spacingAbsolute);
+	void spacingAfterParagraphChange(const double spacingRelative, const double spacingAbsolute);
+	void pageNumberingChange(const WPXPageNumberPosition /* page numbering position */, const uint16_t /* matchedFontPointSize */, const uint16_t /* fontPID */) {}
 	void pageMarginChange(const uint8_t /* side */, const uint16_t /* margin */) {}
 	void pageFormChange(const uint16_t /* length */, const uint16_t /* width */, const WPXFormOrientation /* orientation */) {}
 	void marginChange(const uint8_t side, const uint16_t margin);
 	void paragraphMarginChange(const uint8_t side, const int16_t margin);
 	void indentFirstLineChange(const int16_t offset);
-	void columnChange(const WPXTextColumnType columnType, const uint8_t numColumns, const std::vector<float> &columnWidth,
+	void columnChange(const WPXTextColumnType columnType, const uint8_t numColumns, const std::vector<double> &columnWidth,
 				  const std::vector<bool> &isFixedWidth);
 	void updateOutlineDefinition(const WP6OutlineLocation outlineLocation, const uint16_t outlineHash,
 					     const uint8_t *numberingMethods, const uint8_t tabBehaviourFlag);
@@ -182,7 +189,10 @@ public:
 	void noteOff(const WPXNoteType noteType);
 	void headerFooterGroup(const uint8_t /* headerFooterType */, const uint8_t /* occurenceBits */, const uint16_t /* textPID */) {}
 	void suppressPageCharacteristics(const uint8_t /* suppressCode */) {}
+	void setPageNumber(const uint16_t /* pageNumber */) {}
+        void setPageNumberingType(const WPXNumberingType pageNumberingType);
 	void endDocument() { WPXContentListener::endDocument(); };
+	void endSubDocument() { WPXContentListener::endSubDocument(); };
 
  	void defineTable(const uint8_t position, const uint16_t leftOffset);
 	void addTableColumnDefinition(const uint32_t width, const uint32_t leftGutter, const uint32_t rightGutter,
@@ -194,11 +204,20 @@ public:
 				const RGBSColor * cellBorderColor, const WPXVerticalAlignment cellVerticalAlignment, 
 				const bool useCellAttributes, const uint32_t cellAttributes);
  	void endTable();
+	void boxOn(const uint8_t anchoringType, const uint8_t generalPositioningFlags, const uint8_t horizontalPositioningFlags,
+		const int16_t horizontalOffset, const uint8_t leftColumn, const uint8_t rightColumn,
+		const uint8_t verticalPositioningFlags, const int16_t verticalOffset, const uint8_t widthFlags, const uint16_t width,
+		const uint8_t heightFlags, const uint16_t height, const uint8_t boxContentType, const uint16_t nativeWidth,
+		const uint16_t nativeHeight);
+	void boxOff();
+	void insertGraphicsData(const uint16_t packetId);
+	void insertTextBox(const WP6SubDocument *subDocument);
+	void commentAnnotation(const uint16_t textPID);
 
 	void undoChange(const uint8_t undoType, const uint16_t undoLevel);
 
 protected:
-	void _handleSubDocument(const WPXSubDocument *subDocument, const bool isHeaderFooter, WPXTableList tableList, int nextTableIndice = 0);
+	void _handleSubDocument(const WPXSubDocument *subDocument, WPXSubDocumentType subDocumentType, WPXTableList tableList, int nextTableIndice = 0);
 
 	//void _handleLineBreakElementBegin();
 	void _paragraphNumberOn(const uint16_t outlineHash, const uint8_t level);
