@@ -44,21 +44,25 @@ extern int WP6_FIXED_LENGTH_FUNCTION_GROUP_SIZE[16];
 #define WP6_INDEX_HEADER_NUM_INDICES_POSITION 2
 #define WP6_INDEX_HEADER_INDICES_POSITION 14
 #define WP6_INDEX_HEADER_ELEMENT_CHILD_PACKET_BIT 1
-#define WP6_INDEX_HEADER_GRAPHICS_BOX_STYLE 65
 
+#define WP6_INDEX_HEADER_DOCUMENT_SETTINGS_DESCRIPTOR_POOL 0x02
 #define WP6_INDEX_HEADER_GENERAL_WORDPERFECT_TEXT 0x08
+#define WP6_INDEX_HEADER_NEXT_TOTAL_PAGE_COUNT_DESCRIPTOR_POOL 0x10
+#define WP6_INDEX_HEADER_COMMENT_ANNOTATION 0x1B
 #define WP6_INDEX_HEADER_PRINTER_SELECTION_DESCRIPTOR_POOL 0x23
-#define WP6_INDEX_HEADER_FILL_STYLE 0x43
-#define WP6_INDEX_HEADER_DESIRED_FONT_DESCRIPTOR_POOL 0x55
 #define WP6_INDEX_HEADER_INITIAL_FONT 0x25
 #define WP6_INDEX_HEADER_SYLE_DATA_DESCRIPTOR_POOL 0x30
-#define WP6_INDEX_HEADER_PREFIX_TIME_STAMP_DESCRIPTOR_POOL 0x5E
-#define WP6_INDEX_HEADER_UNDO_INFORMATION_DESCRIPTOR_POOL 0x77 /* WordPerfect 6.1 */
 #define WP6_INDEX_HEADER_SHARED_PERSONAL_FILE_DATES_DESCRIPTOR_POOL 0x34
-#define WP6_INDEX_HEADER_DOCUMENT_SETTINGS_DESCRIPTOR_POOL 0x02
-#define WP6_INDEX_HEADER_NEXT_TOTAL_PAGE_COUNT_DESCRIPTOR_POOL 0x10
-#define WP6_INDEX_HEADER_WORLD_WIDE_WEB_DESCRIPTOR_POOL 0x6D
+#define WP6_INDEX_HEADER_GRAPHICS_FILENAME 0x40
+#define WP6_INDEX_HEADER_GRAPHICS_BOX_STYLE 0x41
+#define WP6_INDEX_HEADER_FILL_STYLE 0x43
 #define WP6_INDEX_HEADER_CHECK_AS_YOU_GO_DESCRIPTOR_POOL 0x4E
+#define WP6_INDEX_HEADER_DESIRED_FONT_DESCRIPTOR_POOL 0x55
+#define WP6_INDEX_HEADER_PREFIX_TIME_STAMP_DESCRIPTOR_POOL 0x5E
+#define WP6_INDEX_HEADER_TABLE_STYLE 0x69
+#define WP6_INDEX_HEADER_GRAPHICS_CACHED_FILE_DATA 0x6F
+#define WP6_INDEX_HEADER_UNDO_INFORMATION_DESCRIPTOR_POOL 0x77 /* WordPerfect 6.1 */
+#define WP6_INDEX_HEADER_WORLD_WIDE_WEB_DESCRIPTOR_POOL 0x6D
 
 #define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY 0x12
 #define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_AUTHOR 5
@@ -102,7 +106,7 @@ extern int WP6_FIXED_LENGTH_FUNCTION_GROUP_SIZE[16];
 #define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_RECORDED_BY 36
 #define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_RECORDED_DATE 37
 #define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_REFERENCE 38
-#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_REVISION_DATE 39	// unused and blank
+#define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_REVISION_DATE 39 /* unused and blank */
 #define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_REVISION_NOTES 40
 #define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_REVISION_NUMBER 41
 #define WP6_INDEX_HEADER_EXTENDED_DOCUMENT_SUMMARY_SECTION 42
@@ -196,6 +200,7 @@ extern int WP6_FIXED_LENGTH_FUNCTION_GROUP_SIZE[16];
 #define WP6_PAGE_GROUP_TOP_MARGIN_SET 0x00
 #define WP6_PAGE_GROUP_BOTTOM_MARGIN_SET 0x01
 #define WP6_PAGE_GROUP_SUPPRESS_PAGE_CHARACTERISTICS 0x02
+#define WP6_PAGE_GROUP_PAGE_NUMBER_POSITION 0x03
 #define WP6_PAGE_GROUP_FORM 0x11
 
 // breaking style here so our symbols aren't (as) ridiculously long
@@ -297,24 +302,13 @@ extern int WP6_FIXED_LENGTH_FUNCTION_GROUP_SIZE[16];
 #define WP6_PARAGRAPH_JUSTIFICATION_FULL_ALL_LINES 0x04
 #define WP6_PARAGRAPH_JUSTIFICATION_RESERVED 0x05
 
-/* Display Number Reference Group */
-#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_USER_DEFINED_ON 0x00
-#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_USER_DEFINED_OFF 0x01
-//#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_CHAPTER_ON 0x08
-//#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_CHAPTER_OFF 0x09
-#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_PARAGRAPH_NUMBER_ON 0x0C
-#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_PARAGRAPH_NUMBER_OFF 0x0D
-#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_FOOTNOTE_NUMBER_ON 0x0E
-#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_FOOTNOTE_NUMBER_OFF 0x0F
-#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_ENDNOTE_NUMBER_ON 0x10
-#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_ENDNOTE_NUMBER_OFF 0x11
-
 /* Character Group */
 #define WP6_CHARACTER_GROUP_SET_ALIGNMENT_CHARACTER 0x00
 #define WP6_CHARACTER_GROUP_COLOR 0x18
 #define WP6_CHARACTER_GROUP_CHARACTER_SHADING_CHANGE 0x19
 #define WP6_CHARACTER_GROUP_FONT_FACE_CHANGE 0x1A
 #define WP6_CHARACTER_GROUP_FONT_SIZE_CHANGE 0x1B
+#define WP6_CHARACTER_GROUP_COMMENT 0x1D
 #define WP6_CHARACTER_GROUP_SET_DOT_LEADER_CHARACTERS 0x1E
 #define WP6_CHARACTER_GROUP_PARAGRAPH_NUMBER_ON 0x32
 #define WP6_CHARACTER_GROUP_PARAGRAPH_NUMBER_OFF 0x33
@@ -339,6 +333,88 @@ extern int WP6_FIXED_LENGTH_FUNCTION_GROUP_SIZE[16];
 #define WP6_HEADER_FOOTER_GROUP_ODD_BIT 0x01
 #define WP6_HEADER_FOOTER_GROUP_EVEN_BIT 0x02
 
+/* Set Number Functions */
+#define WP6_SET_NUMBER_GROUP_SET_USER_DEFINED_COUNTER 0x00
+#define WP6_SET_NUMBER_GROUP_SET_BOX_NUMBER 0x01
+#define WP6_SET_NUMBER_GROUP_SET_PAGE_NUMBER 0x02
+#define WP6_SET_NUMBER_GROUP_SET_SECONDARY_PAGE_NUMBER 0x03
+#define WP6_SET_NUMBER_GROUP_SET_CHAPTER_NUMBER 0x04
+#define WP6_SET_NUMBER_GROUP_SET_VOLUME_NUMBER 0x05
+#define WP6_SET_NUMBER_GROUP_SET_PARAGRAPH_NUMBER 0x06
+#define WP6_SET_NUMBER_GROUP_SET_FOOTNOTE_NUMBER 0x07
+#define WP6_SET_NUMBER_GROUP_SET_ENDNOTE_NUMBER 0x08
+#define WP6_SET_NUMBER_GROUP_SET_LINE_NUMBER 0x09
+
+/* Numbering Functions */
+
+#define WP6_NUMBERING_METHOD_GROUP_SET_USER_DEFINED_COUNTER_NUMBERING_MODE 0x00
+#define WP6_NUMBERING_METHOD_GROUP_SET_BOX_NUMBERING_MODE 0x01
+#define WP6_NUMBERING_METHOD_GROUP_SET_PAGE_NUMBERING_MODE 0x02
+#define WP6_NUMBERING_METHOD_GROUP_SET_SECONDARY_PAGE_NUMBERING_MODE 0x03
+#define WP6_NUMBERING_METHOD_GROUP_SET_CHAPTER_NUMBERING_MODE 0x04
+#define WP6_NUMBERING_METHOD_GROUP_SET_VOLUME_NUMBERING_MODE 0x05
+#define WP6_NUMBERING_METHOD_GROUP_SET_PARAGRAPH_NUMBERING_MODE 0x06
+#define WP6_NUMBERING_METHOD_GROUP_SET_FOOTNOTE_NUMBERING_MODE 0x07
+#define WP6_NUMBERING_METHOD_GROUP_SET_ENDNOTE_NUMBERING_MODE 0x08
+#define WP6_NUMBERING_METHOD_GROUP_SET_LINE_NUMBERING_MODE 0x09
+
+#define WP6_NUMBERING_METHOD_GROUP_PAGE_NUMBERING_ARABIC 0
+#define WP6_NUMBERING_METHOD_GROUP_PAGE_NUMBERING_LOWERCASE 1
+#define WP6_NUMBERING_METHOD_GROUP_PAGE_NUMBERING_UPPERCASE 2
+#define WP6_NUMBERING_METHOD_GROUP_PAGE_NUMBERING_LOWERCASE_ROMAN 3
+#define WP6_NUMBERING_METHOD_GROUP_PAGE_NUMBERING_UPPERCASE_ROMAN 4
+
+/* Display Number Functions */
+
+#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_USER_DEFINED_NUMBER_DISPLAY_ON 0x00
+#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_USER_DEFINED_NUMBER_DISPLAY_OFF 0x01
+#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_BOX_NUMBER_DISPLAY_ON 0x02
+#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_BOX_NUMBER_DISPLAY_OFF 0x03
+#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_PAGE_NUMBER_DISPLAY_ON 0x04
+#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_PAGE_NUMBER_DISPLAY_OFF 0x05
+#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_SECONDARY_PAGE_NUMBER_DISPLAY_ON 0x06
+#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_SECONDARY_PAGE_NUMBER_DISPLAY_OFF 0x07
+#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_CHAPTER_NUMBER_DISPLAY_ON 0x08
+#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_CHAPTER_NUMBER_DISPLAY_OFF 0x09
+#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_VOLUME_NUMBER_DISPLAY_ON 0x0A
+#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_VOLUME_NUMBER_DISPLAY_OFF 0x0B
+#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_PARAGRAPH_NUMBER_DISPLAY_ON 0x0C
+#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_PARAGRAPH_NUMBER_DISPLAY_OFF 0x0D
+#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_FOOTNOTE_NUMBER_DISPLAY_ON 0x0E
+#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_FOOTNOTE_NUMBER_DISPLAY_OFF 0x0F
+#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_ENDNOTE_NUMBER_DISPLAY_ON 0x10
+#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_ENDNOTE_NUMBER_DISPLAY_OFF 0x11
+#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_LINE_NUMBER_DISPLAY_ON 0x12
+#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_LINE_NUMBER_DISPLAY_OFF 0x13
+#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_TOTAL_NUMBER_OF_PAGES_DISPLAY_ON 0x14
+#define WP6_DISPLAY_NUMBER_REFERENCE_GROUP_TOTAL_NUMBER_OF_PAGES_DISPLAY_OFF 0x15
+
+/* Increment Number Functions */
+
+#define WP6_INCREMENT_NUMBER_GROUP_INCREMENT_USER_DEFINED_NUMBER 0x00
+#define WP6_INCREMENT_NUMBER_GROUP_INCREMENT_BOX_NUMBER 0x01
+#define WP6_INCREMENT_NUMBER_GROUP_INCREMENT_PAGE_NUMBER 0x02
+#define WP6_INCREMENT_NUMBER_GROUP_INCREMENT_SECONDARY_PAGE_NUMBER 0x03
+#define WP6_INCREMENT_NUMBER_GROUP_INCREMENT_CHAPTER_NUMBER 0x04
+#define WP6_INCREMENT_NUMBER_GROUP_INCREMENT_VOLUME_NUMBER 0x05
+#define WP6_INCREMENT_NUMBER_GROUP_INCREMENT_PARAGRAPH_NUMBER 0x06
+#define WP6_INCREMENT_NUMBER_GROUP_INCREMENT_FOOTNOTE_NUMBER 0x07
+#define WP6_INCREMENT_NUMBER_GROUP_INCREMENT_ENDNOTE_NUMBER 0x08
+#define WP6_INCREMENT_NUMBER_GROUP_INCREMENT_LINE_NUMBER 0x09
+
+/* Decrement Number Functions */
+
+#define WP6_DECREMENT_NUMBER_GROUP_DECREMENT_USER_DEFINED_NUMBER 0x00
+#define WP6_DECREMENT_NUMBER_GROUP_DECREMENT_BOX_NUMBER 0x01
+#define WP6_DECREMENT_NUMBER_GROUP_DECREMENT_PAGE_NUMBER 0x02
+#define WP6_DECREMENT_NUMBER_GROUP_DECREMENT_SECONDARY_PAGE_NUMBER 0x03
+#define WP6_DECREMENT_NUMBER_GROUP_DECREMENT_CHAPTER_NUMBER 0x04
+#define WP6_DECREMENT_NUMBER_GROUP_DECREMENT_VOLUME_NUMBER 0x05
+#define WP6_DECREMENT_NUMBER_GROUP_DECREMENT_PARAGRAPH_NUMBER_0x06
+#define WP6_DECREMENT_NUMBER_GROUP_DECREMENT_FOOTNOTE_NUMBER 0x07
+#define WP6_DECREMENT_NUMBER_GROUP_DECREMENT_ENDNOTE_NUMBER 0x08
+#define WP6_DECREMENT_NUMBER_GROUP_DECREMENT_LINE_NUMBER 0x09
+
 /* Column Group */
 #define WP6_COLUMN_GROUP_LEFT_MARGIN_SET 0
 #define WP6_COLUMN_GROUP_RIGHT_MARGIN_SET 1
@@ -367,6 +443,24 @@ extern int WP6_FIXED_LENGTH_FUNCTION_GROUP_SIZE[16];
 #define WP6_ATTRIBUTE_SMALL_CAPS 15
 #define WP6_ATTRIBUTE_BLINK 16
 
+/* Box Group */
+#define WP6_BOX_GROUP_CHARACTER_ANCHORED_BOX 0x00
+#define WP6_BOX_GROUP_PARAGRAPH_ANCHORED_BOX 0x01
+#define WP6_BOX_GROUP_PAGE_ANCHORED_BOX 0x02
+#define WP6_BOX_GROUP_GRAPHICS_RULE 0x03
+
+#define WP6_BOX_GROUP_BOX_COUNTER_DATA_BIT 0x8000
+#define WP6_BOX_GROUP_BOX_POSITIONING_DATA_BIT 0x4000
+#define WP6_BOX_GROUP_BOX_CONTENT_DATA_BIT 0x2000
+#define WP6_BOX_GROUP_BOX_CAPTION_DATA_BIT 0x1000
+#define WP6_BOX_GROUP_BOX_BORDER_DATA_BIT 0x0800
+#define WP6_BOX_GROUP_BOX_FILL_DATA_BIT 0x0400
+#define WP6_BOX_GROUP_BOX_BOX_WRAPPING_DATA_BIT 0x0200
+#define WP6_BOX_GROUP_BOX_BOX_HYPERTEXT_WRAPPING_DATA_BIT 0x0100
+#define WP6_BOX_GROUP_BOX_HTML_BIT 0x0080
+#define WP6_BOX_GROUP_BOX_GROUPING_DATA_BIT 0x0040
+#define WP6_BOX_GROUP_BOX_DRAW_OBJECT_DATA_BIT 0x0020
+
 /* Tab Group */
 // these elements are defined in terms of bits 3-8 of the tab group
 #define WP6_TAB_GROUP_BACK_TAB 0x00
@@ -382,6 +476,56 @@ extern int WP6_FIXED_LENGTH_FUNCTION_GROUP_SIZE[16];
 #define WP6_TAB_GROUP_RIGHT_TAB 0x12
 #define WP6_TAB_GROUP_DECIMAL_TAB 0x1A
 
+/* Platform Group */
+#define WP6_PLATFORM_GROUP_DYNAMIC_DATA_EXCHANGE_START 0x00
+#define WP6_PLATFORM_GROUP_DYNAMIC_DATA_EXCHANGE_END 0x01
+#define WP6_PLATFORM_GROUP_PUBLISH_DATA_START 0x02
+#define WP6_PLATFORM_GROUP_PUBLISH_DATA_END 0x03
+#define WP6_PLATFORM_GROUP_SUBSCRIBE_DATA_START 0x04
+#define WP6_PLATFORM_GROUP_SUBSCRIBE_DATA_END 0x05
+#define WP6_PLATFORM_GROUP_SUBTITLE_MARK 0x06
+#define WP6_PLATFORM_GROUP_SUBTITLE_TEXT 0x07
+#define WP6_PLATFORM_GROUP_SUBTITLE_OPTION 0x08
+#define WP6_PLATFORM_GROUP_SUBTITLE_TEXT_OPTIONS 0x09
+#define WP6_PLATFORM_GROUP_SUPPRESS_SUBTITLE Ox0A
+#define WP6_PLATFORM_GROUP_THIRD_PARTY 0x0B
+#define WP6_PLATFORM_GROUP_PEN_FOREGROUND_COLOR_32BIT 0x0C
+#define WP6_PLATFORM_GROUP_PEN_FOREGROUND_COLOR_64BIT 0x0D
+#define WP6_PLATFORM_GROUP_PEN_BACKGROUND_COLOR_32BIT 0x0E
+#define WP6_PLATFORM_GROUP_PEN_BACKGROUND_COLOR_64BIT 0x0F
+#define WP6_PLATFORM_GROUP_PEN_STYLE 0x10
+#define WP6_PLATFORM_GROUP_PEN_PATTERN 0x11
+#define WP6_PLATFORM_GROUP_PEN_SIZE_16BIT 0x12
+#define WP6_PLATFORM_GROUP_PEN_SIZE_32BIT 0x13
+#define WP6_PLATFORM_GROUP_BRUSH_GRADIENT_32BIT 0x14
+#define WP6_PLATFORM_GROUP_BRUSH_GRADIENT_64BIT 0x15
+#define WP6_PLATFORM_GROUP_BRUSH_FOREGROUND_COLOR_32BIT 0x16
+#define WP6_PLATFORM_GROUP_BRUSH_FOREGROUND_COLOR_64BIT 0x17
+#define WP6_PLATFORM_GROUP_BRUSH_BACKGROUND_COLOR_32BIT 0x18
+#define WP6_PLATFORM_GROUP_BRUSH_BACKGROUND_COLOR_64BIT 0x19
+#define WP6_PLATFORM_GROUP_BRUSH_PATTERN 0x1A
+#define WP6_PLATFORM_GROUP_MISCELLANEOUS_ATTRIBUTES 0x1B
+#define WP6_PLATFORM_GROUP_PRESENTATIONS_SLIDE_OPTIONS 0x1C
+#define WP6_PLATFORM_GROUP_PRESENTATIONS_ORGANIZATION_CHART_NODE_OPTIONS 0x1D
+#define WP6_PLATFORM_GROUP_HIDE_FOR_CONVERSIONS 0x1E
+#define WP6_PLATFORM_GROUP_HIDE_OFF_FOR_CONVERSIONS 0x1F
+#define WP6_PLATFORM_GROUP_SGML_START_TAG 0x20
+#define WP6_PLATFORM_GROUP_SGML_START_TAG_END 0x21
+#define WP6_PLATFORM_GROUP_SGML_END_TAG 0x22
+#define WP6_PLATFORM_GROUP_SGML_END_TAG_END 0x23
+#define WP6_PLATFORM_GROUP_SGML_CHARACTER_REFERENCE 0x24
+#define WP6_PLATFORM_GROUP_SGML_END_CHARACTER_REFERENCE 0x25
+#define WP6_PLATFORM_GROUP_SGML_ENTITY_REFERENCE 0x26
+#define WP6_PLATFORM_GROUP_SGML_END_ENTITY_REFERENCE 0x27
+#define WP6_PLATFORM_GROUP_SGML_IGNORE_ON 0x28
+#define WP6_PLATFORM_GROUP_SGML_IGNORE_OFF 0x29
+#define WP6_PLATFORM_GROUP_SGML_MARKED_SECTION_ON 0x2A
+#define WP6_PLATFORM_GROUP_SGML_MARKED_SECTION_OFF 0x2B
+#define WP6_PLATFORM_GROUP_SGML_PROCESSING_INSTRUCTION_ON 0x2C
+#define WP6_PLATFORM_GROUP_SGML_PROCESSING_INSTRUCTION_OFF 0x2D
+
+/* Set number group */
+#define WP6_SET_NUMBER_GROUP_SET_PAGE_NUMBER 0x02
 
 /* Extended Character */
 #define WP6_MULTINATIONAL_CHARACTER_SET 1
