@@ -31,11 +31,11 @@
 class WP6UnsupportedFixedLengthGroup : public WP6FixedLengthGroup
 {
  public:
-	WP6UnsupportedFixedLengthGroup(WPXInputStream *input, uint8_t groupID);	
+	WP6UnsupportedFixedLengthGroup(WPXInputStream *input, WPXEncryption *encryption, uint8_t groupID);	
 	void parse(WP6Listener * /* listener */) {}
 	
  protected:
-	void _readContents(WPXInputStream * /* input */) {}
+	void _readContents(WPXInputStream * /* input */, WPXEncryption * /* encryption */) {}
 };
 
 #endif /* WP6UNSUPPORTEDFIXEDLENGTHGROUP_H */
