@@ -1,5 +1,5 @@
 /* libwpd
- * Copyright (C) 2002 William Lachance (william.lachance@sympatico.ca)
+ * Copyright (C) 2002 William Lachance (wrlach@gmail.com)
  * Copyright (C) 2002-2003 Marc Maurer (uwog@uwog.net)
  *  
  * This library is free software; you can redistribute it and/or
@@ -26,23 +26,26 @@
 #ifndef WPXHEADER_H
 #define WPXHEADER_H
 
-#include <stdlib.h>
 #include "WPXStream.h"
+#include "libwpd_types.h"
+
+class WPXEncryption;
 
 class WPXHeader
 {
  public:	
-	WPXHeader(WPXInputStream *input, uint32_t documentOffset, uint8_t productType, uint8_t fileType, uint8_t majorVersion, uint8_t minorVersion, uint16_t documentEncryption);
+	WPXHeader(WPXInputStream *input, WPXEncryption *encryption, uint32_t documentOffset, uint8_t productType,
+		uint8_t fileType, uint8_t majorVersion, uint8_t minorVersion, uint16_t documentEncryption);
 	virtual ~WPXHeader();
 
-	static WPXHeader * constructHeader(WPXInputStream *input);
+	static WPXHeader * constructHeader(WPXInputStream *input, WPXEncryption *encryption);
 		
-	const uint32_t getDocumentOffset() const { return m_documentOffset; }
-	const uint8_t getProductType() const { return m_productType; }
-	const uint8_t getFileType() const { return m_fileType; }
-	const uint8_t getMajorVersion() const { return m_majorVersion; }
-	const uint8_t getMinorVersion() const { return m_minorVersion; }
-	const uint16_t getDocumentEncryption() const { return m_documentEncryption; }
+	uint32_t getDocumentOffset() const { return m_documentOffset; }
+	uint8_t getProductType() const { return m_productType; }
+	uint8_t getFileType() const { return m_fileType; }
+	uint8_t getMajorVersion() const { return m_majorVersion; }
+	uint8_t getMinorVersion() const { return m_minorVersion; }
+	uint16_t getDocumentEncryption() const { return m_documentEncryption; }
 
  private:	
 	uint32_t m_documentOffset;
