@@ -1,5 +1,5 @@
 /* libwpd
- * Copyright (C) 2002 William Lachance (william.lachance@sympatico.ca)
+ * Copyright (C) 2002 William Lachance (wrlach@gmail.com)
  * Copyright (C) 2002 Marc Maurer (uwog@uwog.net)
  *  
  * This library is free software; you can redistribute it and/or
@@ -26,7 +26,12 @@
 #ifndef LIBWPD_H
 #define LIBWPD_H
 
-#include "WPXHLListenerImpl.h"
+#define LIBWPD_VERSION_MAJOR     0
+#define LIBWPD_VERSION_MINOR     9
+#define LIBWPD_VERSION_REVISION  0
+#define LIBWPD_VERSION_STRING    "0.9.0"
+
+#include "WPXDocumentInterface.h"
 #include "WPDocument.h"
 
 #endif /* LIBWPD_H */
